package hanalife.b.bd.br.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import hanalife.b.bd.br.mapper.ExmpConsCostProcMapper;
import hanalife.b.bd.br.mapper.dto.ExmpConsCostProcO01DTO;
import hanalife.b.bd.br.service.ExmpConsCostProcService;
import hanalife.b.bd.br.vo.ExmpConsCostObjVO;
import hanalife.b.bd.br.vo.ExmpConsCostProcInVO;
import hanalife.b.bd.br.vo.ExmpConsCostProcOutVO;
import hanalife.c.co.iEnum.MsgEnum;

/**
 * <pre>
 * 프로젝트명   : 하나생명 가계대출 시스템(DOBSCAP)
 * 클래스명     : ExmpConsCostProcServiceImpl.java Class
 * 클래스설명   : 면제품의 수납 조회, 수정 처리 (ExmpConsCostProc) Service Implements
 * 작성일자     : 2023. 3. 14.
 * 작성자명     : 70321 / KimMinSeok / 가계대출 시스템 구축
 * Copyrightⓒ2022 Hana Life Insurance, All Rights Reserved.
 * -----------------------------------------------------------------------------
 * Modification Information
 * -----------------------------------------------------------------------------
 * [  수정일자  ][수정자][변경사유-수정내용]
 * -----------------------------------------------------------------------------
 * [2023. 3. 14.][70321][최초작성]
 * -----------------------------------------------------------------------------
 * 서비스구성 : readExmpConsCost : 면제품의내역 조회
 *          calcExmpConsCost : 면제품의내역 수정
 * </pre>
 */
@Service("ExmpConsCostProcService")
public class ExmpConsCostProcServiceImpl implements ExmpConsCostProcService {

	private static final Logger LOGGER = LoggerFactory.getLogger(ExmpConsCostProcServiceImpl.class);

    @Autowired
    private ExmpConsCostProcMapper exmpConsCostProcMapper;

    /** 공통오류코드 및 오류메시지 내용 전역 변수 선언 */
    private MsgEnum msgEnum;

	private boolean isValidVO(ExmpConsCostProcInVO leglActnCostVO) {
		if (leglActnCostVO == null || leglActnCostVO.getListExmpConsCostObjVO() == null) {
			return false;
		}
		return true;
	}

	private boolean debugFlag = false;

	/**
	 * <p>메소드명      : readExmpConsCost : 면제품의내역 조회</p>
	 * <p>작성일자      : 2023. 3. 14.
	 * <p>작성자(author): 70321 / MinSeok
	 * <p>설명          : 면제품의내역을 조회 한다.</p>
	 * <p>메소드인수    : ExmpConsCostProcInVO </p>
	 * <p>메소드리턴값  : ExmpConsCostProcOutVO</p>
	 * <p>예외처리      : Exception</p>
	 */
	@Override
	public ExmpConsCostProcOutVO readExmpConsCost(ExmpConsCostProcInVO inVo) throws Exception {

		/** 면제품의내역 조회 서비스 결과값 객체 생성 */
		ExmpConsCostProcOutVO resultVo = new ExmpConsCostProcOutVO();

		/** Validation VO */
		if (!isValidVO(inVo)) {
			resultVo.setRsltCd(MsgEnum.DATA_ZERO_READ_EXMP.getErrorCd());
			resultVo.setRsltCntn(MsgEnum.DATA_ZERO_READ_EXMP.getDescription());
			if (debugFlag) {
				LOGGER.debug("[면제품의내역 조회 결과 정보]");
				LOGGER.debug(resultVo.toString());
			}
			return resultVo;
		}

		if (debugFlag) {
			LOGGER.debug("[면제품의내역 조회 요청 정보]");
			LOGGER.debug(inVo.toString());
		}

		/** 파라미터로 받은 조회 조건 리스트 */
		List<ExmpConsCostObjVO> exmpConsCostObjVOList = inVo.getListExmpConsCostObjVO();

		try {

			for(ExmpConsCostObjVO exmpConsCostObjVO : exmpConsCostObjVOList) {

				/** 면제품의내역에 대한 조회 조건 입력값 객체 생성 및 대입 수행 */
				ExmpConsCostProcO01DTO targetDto = new ExmpConsCostProcO01DTO();

				targetDto.setLoanNo(exmpConsCostObjVO.getLoanNo()); /** 대출_번호 */
				targetDto.setLoanSeq(exmpConsCostObjVO.getLoanSeq()); /** 대출_순번 */
				targetDto.setConsShNo(exmpConsCostObjVO.getConsShNo()); /** 품의_번호 */

				/** 면제품의내역에 대한 조회 쿼리 Mapper 호출 수행 */
				List<ExmpConsCostProcO01DTO> resultDTO = exmpConsCostProcMapper.selectExmpConsMng(targetDto);

				/** 결과 값에 따른 결과코드 및 메시지 생성하기 위한 변수 */
				boolean chkOver = false;
				int next = 0;
				int size = resultDTO.size();

				/** rs.next()와 동일 로직 및 결과 리스트의 값 존재 확인 */
				if(next < size) {
					next++;
					chkOver = true;
				}

				/** rs.next()와 동일 로직 및 결과 리스트가 여러개 일 경우 체크 */
				if(next < size) {
					resultVo.setRsltCd(MsgEnum.TOO_MANY_READ_EXMP.getErrorCd());
					resultVo.setRsltCntn(MsgEnum.TOO_MANY_READ_EXMP.getDescription());
					next++;
				}

				/** 결과 리스트의 값이 존재하지 않는 경우와 정상인 케이스 감별 */
				if (chkOver) {
					if (!resultVo.getRsltCd().equals("N03")) {
						resultVo.setRsltCd(MsgEnum.SUCCESS_READ_EXMP.getErrorCd());
						resultVo.setRsltCntn(MsgEnum.SUCCESS_READ_EXMP.getDescription());
					}
				} else {
					resultVo.setRsltCd(MsgEnum.NOT_FOUND_READ_EXMP.getErrorCd());
					resultVo.setRsltCntn(MsgEnum.NOT_FOUND_READ_EXMP.getDescription());
				}

				/** 따로 사용하지 않는 exmpConsCostObjVO List이므로 주석 처리 */
//				for (ExmpConsCostProcO01DTO sourceIO : resultDTO) {
//					ExmpConsCostObjVO targetIO = new ExmpConsCostObjVO();
//
//					targetIO.setConsShNo(sourceIO.getConsShNo()); /** 품의_번호 */
//					targetIO.setConsDt(sourceIO.getConsDt()); /** 품의_일자 */
//					targetIO.setExmpLoanPamt(sourceIO.getExmpLoanPamt()); /** 면제_대출_원금 */
//					targetIO.setExmpLoanInt(sourceIO.getExmpLoanInt()); /** 면제_대출_이자 */
//					targetIO.setExmpDlyInt(sourceIO.getExmpDlyInt()); /** 면제_연체_이자 */
//					targetIO.setExmpSpayAmt(sourceIO.getExmpSpayAmt()); /** 면제_가지급금 */
//					targetIO.setExmpDlyFee(sourceIO.getExmpDlyFee()); /** 면제_연체료 */
//					targetIO.setExmpTrtFee(sourceIO.getExmpTrtFee()); /** 면제_취급_수수료 */
//					targetIO.setExmpMidRmbrFee(sourceIO.getExmpMidRmbrFee()); /** 면제_중도_상환_수수료 */
//					targetIO.setExmpAmt(sourceIO.getExmpAmt()); /** 면제_금액 */
//					targetIO.setProcDivCd("N01"); /** 처리_구분 */
//
//					resultList.add(targetIO);
//				}
			}

		}catch (Exception e) {
			resultVo.setRsltCd(MsgEnum.ERR_SYSTEM_READ_EXMP.getErrorCd());
			resultVo.setRsltCntn(e.getMessage());
		}

		/** 결과 코드, 메시지 리턴 */
		return resultVo;

	}

	/**
	 * <p>메소드명      : calcExmpConsCost - 면제품의내역 수정</p>
	 * <p>작성일자      : 2023. 3. 14.
	 * <p>작성자(author): 70321 / MinSeok
	 * <p>설명          : 면제품의내역 수정</p>
	 * <p>메소드인수    : ExmpConsCostProcInVO </p>
	 * <p>메소드리턴값  : ExmpConsCostProcOutVO</p>
	 * <p>예외처리      : Exception</p>
	 */
	@Override
	public ExmpConsCostProcOutVO calcExmpConsCost(ExmpConsCostProcInVO inVo) throws Exception {

		/** 면제품의내역 수정 서비스 결과값 객체 생성 */
		ExmpConsCostProcOutVO resultVo = new ExmpConsCostProcOutVO();

		/** Validation VO */
		if (!isValidVO(inVo)) {
			resultVo.setRsltCd(msgEnum.DATA_ZERO_CALC_EXMP.getErrorCd());
			resultVo.setRsltCntn(msgEnum.DATA_ZERO_CALC_EXMP.getDescription());
			if (debugFlag) {
				LOGGER.debug("[면제품의내역 수납 반영 결과 정보]");
				LOGGER.debug(resultVo.toString());
			}
			return resultVo;
		}

		if (debugFlag) {
			LOGGER.debug("[면제품의내역 수납 반영 정보]");
			LOGGER.debug(inVo.toString());
		}

		/** 파라미터로 받은 조회 조건 리스트 */
		List<ExmpConsCostObjVO> exmpConsCostObjVOList = inVo.getListExmpConsCostObjVO();

		try {

			for(ExmpConsCostObjVO exmpConsCostObjVO : exmpConsCostObjVOList) {

				/** 면제품의내역에 대한 조회 조건 입력값 객체 생성 및 대입 수행 */
				ExmpConsCostProcO01DTO targetDto = new ExmpConsCostProcO01DTO();

				targetDto.setLoanNo(exmpConsCostObjVO.getLoanNo()); /** 대출_번호 */
				targetDto.setLoanSeq(exmpConsCostObjVO.getLoanSeq()); /** 대출_순번 */
				targetDto.setConsShNo(exmpConsCostObjVO.getConsShNo()); /** 품의_번호 */

				/** 면제품의내역에 대한 조회 쿼리 Mapper 호출 수행 */
				List<ExmpConsCostProcO01DTO> resultDTO = new ArrayList<ExmpConsCostProcO01DTO>();

				if (exmpConsCostObjVO.getProcDivCd().trim().equals("U")) {// 수납시

					resultDTO = exmpConsCostProcMapper.selectExmpConsMngCount1(targetDto);

					/** 수정할 리스트가 존재하면 면제품의내역에 대한 수정 쿼리 Mapper 호출 수행 */
					if(resultDTO != null && resultDTO.isEmpty()) {

						targetDto.setRcptAplyDt(exmpConsCostObjVO.getRcptAplyDt()); /** 수납_반영_일자 */
						targetDto.setRcptAplyEmpNo(exmpConsCostObjVO.getRcptAplyEmpNo()); /** 수납_반영_사원_번호 */
						targetDto.setLastProcEmpNo(exmpConsCostObjVO.getLastProcEmpNo()); /** 최종_처리_사원_번호 */
						targetDto.setLastProcDt(exmpConsCostObjVO.getLastProcDt()); /** 최종_처리_일자 */
						targetDto.setLastProcTm(exmpConsCostObjVO.getLastProcTm()); /** 최종_처리_시각 */

						this.exmpConsCostProcMapper.updateExmpConsMng(targetDto);

						resultVo.setRsltCd(msgEnum.SUCCESS_CALC_EXMP.getErrorCd());
						resultVo.setRsltCntn(msgEnum.SUCCESS_CALC_EXMP.getDescription());

					} else {
						resultVo.setRsltCd(msgEnum.NOT_FOUND_CALC_EXMP.getErrorCd());
						resultVo.setRsltCntn(msgEnum.NOT_FOUND_CALC_EXMP.getDescription());
					}

				} else {// 취소시
					resultDTO = exmpConsCostProcMapper.selectExmpConsMngCount2(targetDto);

					/** 수정할 리스트가 존재하면 면제품의내역에 대한 수정 쿼리 Mapper 호출 수행 */
					if(resultDTO != null && resultDTO.isEmpty()) {

//						targetDto.setRcptAplyDt(""); /** 수납_반영_일자 */
//						targetDto.setRcptAplyEmpNo(""); /** 수납_반영_사원_번호 */
						targetDto.setLastProcEmpNo(exmpConsCostObjVO.getLastProcEmpNo()); /** 최종_처리_사원_번호 */
						targetDto.setLastProcDt(exmpConsCostObjVO.getLastProcDt()); /** 최종_처리_일자 */
						targetDto.setLastProcTm(exmpConsCostObjVO.getLastProcTm()); /** 최종_처리_시각 */

						this.exmpConsCostProcMapper.updateExmpConsMng(targetDto);

						resultVo.setRsltCd(msgEnum.SUCCESS_CALC_EXMP.getErrorCd());
						resultVo.setRsltCntn(msgEnum.SUCCESS_CALC_EXMP.getDescription());

					} else {
						resultVo.setRsltCd(msgEnum.NOT_FOUND_CALC_EXMP.getErrorCd());
						resultVo.setRsltCntn(msgEnum.NOT_FOUND_CALC_EXMP.getDescription());
					}
				}

			}
		} catch (Exception e) {
			resultVo.setRsltCd(msgEnum.ERR_SYSTEM_CALC_EXMP.getErrorCd());
			resultVo.setRsltCntn(e.getMessage());
		}

		/** 로직 수행 후 수정 반영 결과에 따른 결과 코드 및 메시지 리턴 */
		return resultVo;

	}
}
