package hanalife.batch.util.jdbc;

import java.io.InputStream;
import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.log4j.Logger;

/**
 * <pre><code>
 *     ResultSet rs = null;
 *     NpPreparedStatement pstmt = null;
 *     try {
 *         String sql = "select clas1, max(CD_DESC_KOR_NM) CD_DESC_KOR_NM\n" +
 *                 "  from DBSCOMM.CT_COMM_CD_DESC\n" +
 *                 "where cd_kind_no = :cdKindNo \n" +
 *                 "group by clas1";
 *
 *         pstmt = NpPreparedStatement.createNpPreparedStatement(con, sql);
 *         pstmt.setString("cdKindNo", "R00015");
 *
 *         rs = pstmt.executeQuery();
 *
 *         while(rs.next()) {
 *             System.out.println(">> : " + RUtil.trim(rs.getString("CD_DESC_KOR_NM")));
 *         }
 *
 *     } finally {
 *         DaoUtil.closeResultSet(rs);
 *         DaoUtil.closeStatement(pstmt);
 *     }
 * </code></pre>
 */
public class NpPreparedStatement extends DelegatingPreparedStatement {
    private final List<String> orderedParameters;
    private String sqlTemplate;
    private Logger log = Logger.getLogger(NpPreparedStatement.class.getName());
    public static boolean isQueryLog = true;

    /**
     * Class to hold the results of SQL parsing (used by factory methods)
     */
    static class ParseResult {
        private String sql;
        private List<String> orderedParameters;

        public ParseResult(String sql, List<String> orderedParameters) {
            this.sql = sql;
            this.orderedParameters = orderedParameters;
        }

        public String getSql() {
            return sql;
        }
        public List<String> getOrderedParameters() {
            return orderedParameters;
        }
    }

    /**
     * Parse the query string containing named parameters and result a parse result, which holds
     * the parsed sql (named parameters replaced by standard '?' parameters and an ordered list of the
     * named parameters.
     *
     * SQL parsing code borrowed from Adam Crume. Thanks Adam.
     * See <a href="http://www.javaworld.com/article/2077706/core-java/named-parameters-for-preparedstatement.html?page=2">http://www.javaworld.com/article/2077706/core-java/named-parameters-for-preparedstatement.html?page=2</a>
     *
     * @param query Query containing named parameters
     * @return ParseResult
     */
    public static ParseResult parse(String query) {
        List<String> orderedParameters = new ArrayList<String>();
        int length = query.length();
        StringBuffer parsedQuery = new StringBuffer(length);
        boolean inSingleQuote = false;
        boolean inDoubleQuote = false;
        boolean inSingleLineComment = false;
        boolean inMultiLineComment = false;
        boolean inDoubleСolon = false;

        for (int i = 0; i < length; i++) {
            char c = query.charAt(i);
            if (inSingleQuote) {
                if (c == '\'') {
                    inSingleQuote = false;
                }
            } else if (inDoubleQuote) {
                if (c == '"') {
                    inDoubleQuote = false;
                }
            } else if (inMultiLineComment) {
                if (c == '*' && query.charAt(i + 1) == '/') {
                    inMultiLineComment = false;
                }
            } else if (inDoubleСolon) {
            	if (!Character.isJavaIdentifierPart(c)) {
            		inDoubleСolon = false;
                }
            } else if (inSingleLineComment) {
                if (c == '\n') {
                    inSingleLineComment = false;
                }
            } else {
                if (c == '\'') {
                    inSingleQuote = true;
                } else if (c == '"') {
                    inDoubleQuote = true;
                } else if (c == '/' && query.charAt(i + 1) == '*') {
                    inMultiLineComment = true;
                } else if (c == '-' && query.charAt(i + 1) == '-') {
                    inSingleLineComment = true;
                } else if (c == ':' && query.charAt(i + 1) == ':') {
                	inDoubleСolon = true;
                } else if (c == ':' && i + 1 < length && Character.isJavaIdentifierStart(query.charAt(i + 1))) {
                    int j = i + 2;
                    while (j < length && Character.isJavaIdentifierPart(query.charAt(j))) {
                        j++;
                    }
                    String name = query.substring(i + 1, j);
                    orderedParameters.add(name);
                    c = '?'; // replace the parameter with a question mark
                    i += name.length(); // skip past the end if the parameter
                }
            }
            parsedQuery.append(c);
        }
        return new ParseResult(parsedQuery.toString(), orderedParameters);
    }

    //factory methods for all possible PreparedStatement constructors
    public static NpPreparedStatement createNpPreparedStatement(Connection conn, String sql) throws SQLException {
        ParseResult parseResult = parse(sql);
        return new NpPreparedStatement(conn.prepareStatement(parseResult.getSql()), parseResult.getOrderedParameters(), parseResult.getSql());
    }

    public static NpPreparedStatement createNpPreparedStatement(Connection conn, String sql,
                                                                int autoGeneratedKeys) throws SQLException {
        ParseResult parseResult = parse(sql);
        return new NpPreparedStatement(conn.prepareStatement(parseResult.getSql(), autoGeneratedKeys),
                parseResult.getOrderedParameters(), parseResult.getSql());
    }

    public static NpPreparedStatement createNpPreparedStatement(Connection conn, String sql,
                                                                int[] columnIndexes) throws SQLException {
        ParseResult parseResult = parse(sql);
        return new NpPreparedStatement(conn.prepareStatement(parseResult.getSql(), columnIndexes),
                parseResult.getOrderedParameters(), parseResult.getSql());
    }

    public static NpPreparedStatement createNpPreparedStatement(Connection conn, String sql,
                                                                String[] columnNames) throws SQLException {
        ParseResult parseResult = parse(sql);
        return new NpPreparedStatement(conn.prepareStatement(parseResult.getSql(), columnNames),
                parseResult.getOrderedParameters(), parseResult.getSql());
    }

    public static NpPreparedStatement createNpPreparedStatement(Connection conn, String sql,
                                                                int resultSetType, int resultSetConcurrency) throws SQLException {
        ParseResult parseResult = parse(sql);
        return new NpPreparedStatement(conn.prepareStatement(parseResult.getSql(), resultSetType, resultSetConcurrency),
                parseResult.getOrderedParameters(), parseResult.getSql());
    }

    public static NpPreparedStatement createNpPreparedStatement(Connection conn, String sql,
                                                                int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
        ParseResult parseResult = parse(sql);
        return new NpPreparedStatement(conn.prepareStatement(parseResult.getSql(), resultSetType, resultSetConcurrency, resultSetHoldability),
                parseResult.getOrderedParameters(), parseResult.getSql());
    }

    /**
     * Private constructor (use factory methods)
     * @param delegate PreparedStatement delegate
     * @param orderedParameters Ordered list of named parameters produced during parsing
     */
    private NpPreparedStatement(PreparedStatement delegate, List<String> orderedParameters, String sql) {
        super(delegate);
        this.orderedParameters = orderedParameters;
        this.sqlTemplate = sql;
    }

    public boolean hasNamedParameters() {
        return !orderedParameters.isEmpty();
    }

    private Collection<Integer> getParameterIndexes(String parameter) {
        Collection<Integer> indexes = new ArrayList<Integer>();
        for (int i = 0; i < orderedParameters.size(); i++) {
            if (orderedParameters.get(i).equals(parameter)) {
                //add i + 1, since all indexes ever are 0 based EXCEPT JDBC PARAMS WHYYYYY
                indexes.add(i + 1);
            }
        }
        if (indexes.isEmpty()) {
            throw new IllegalArgumentException(String.format("SQL statement doesn't contain the parameter '%s'",
                    parameter));
        }
        return indexes;
    }

    public void setNull(String parameter, int sqlType) throws SQLException {
        for (Integer i : getParameterIndexes(parameter)) {
            getDelegate().setNull(i, sqlType);
            saveQueryParamValue(i, sqlType);
        }
    }

    public void setBoolean(String parameter, boolean x) throws SQLException {
        for (Integer i : getParameterIndexes(parameter)) {
            getDelegate().setBoolean(i, x);
            saveQueryParamValue(i, x);
        }
    }

    public void setByte(String parameter, byte x) throws SQLException {
        for (Integer i : getParameterIndexes(parameter)) {
            getDelegate().setByte(i, x);
            saveQueryParamValue(i, x);
        }
    }

    public void setShort(String parameter, short x) throws SQLException {
        for (Integer i : getParameterIndexes(parameter)) {
            getDelegate().setShort(i, x);
            saveQueryParamValue(i, x);
        }
    }

    public void setInt(String parameter, int x) throws SQLException {
        for (Integer i : getParameterIndexes(parameter)) {
            getDelegate().setInt(i, x);
            saveQueryParamValue(i, x);
        }
    }

    public void setLong(String parameter, long x) throws SQLException {
        for (Integer i : getParameterIndexes(parameter)) {
            getDelegate().setLong(i, x);
            saveQueryParamValue(i, x);
        }
    }

    public void setFloat(String parameter, float x) throws SQLException {
        for (Integer i : getParameterIndexes(parameter)) {
            getDelegate().setFloat(i, x);
            saveQueryParamValue(i, x);
        }
    }

    public void setDouble(String parameter, float x) throws SQLException {
        for (Integer i : getParameterIndexes(parameter)) {
            getDelegate().setDouble(i, x);
            saveQueryParamValue(i, x);
        }
    }

    public void setDouble(String parameter, double x) throws SQLException {
        for (Integer i : getParameterIndexes(parameter)) {
            getDelegate().setDouble(i, x);
            saveQueryParamValue(i, x);
        }
    }

    public void setBigDecimal(String parameter, BigDecimal x) throws SQLException {
        for (Integer i : getParameterIndexes(parameter)) {
            getDelegate().setBigDecimal(i, x);
            saveQueryParamValue(i, x);
        }
    }

    public void setString(String parameter, String x) throws SQLException {
        for (Integer i : getParameterIndexes(parameter)) {
            getDelegate().setString(i, x);
            saveQueryParamValue(i, x);
        }
    }

    public void setBytes(String parameter, byte[] x) throws SQLException {
        for (Integer i : getParameterIndexes(parameter)) {
            getDelegate().setBytes(i, x);
            saveQueryParamValue(i, x);
        }
    }

    public void setDate(String parameter, Date x) throws SQLException {
        for (Integer i : getParameterIndexes(parameter)) {
            getDelegate().setDate(i, x);
            saveQueryParamValue(i, x);
        }
    }

    public void setTime(String parameter, Time x) throws SQLException {
        for (Integer i : getParameterIndexes(parameter)) {
            getDelegate().setTime(i, x);
            saveQueryParamValue(i, x);
        }
    }

    public void setTimestamp(String parameter, Timestamp x) throws SQLException {
        for (Integer i : getParameterIndexes(parameter)) {
            getDelegate().setTimestamp(i, x);
            saveQueryParamValue(i, x);
        }
    }

    public void setAsciiStream(String parameter, InputStream x) throws SQLException {
        for (Integer i : getParameterIndexes(parameter)) {
            getDelegate().setAsciiStream(i, x);
            saveQueryParamValue(i, x);
        }
    }

    @Deprecated
    public void setUnicodeStream(String parameter, InputStream x, int length) throws SQLException {
        for (Integer i : getParameterIndexes(parameter)) {
            getDelegate().setUnicodeStream(i, x, length);
        }
    }

    public void setBinaryStream(String parameter, InputStream x, int length) throws SQLException {
        for (Integer i : getParameterIndexes(parameter)) {
            getDelegate().setBinaryStream(i, x, length);
        }
    }

    public void setObject(String parameter, Object x, int targetSqlType, int scale) throws SQLException {
        for (Integer i : getParameterIndexes(parameter)) {
            getDelegate().setObject(i, x, targetSqlType, scale);
        }
    }

    public void setObject(String parameter, Object x, int targetSqlType) throws SQLException {
        for (Integer i : getParameterIndexes(parameter)) {
            getDelegate().setObject(i, x, targetSqlType);
        }
    }

    public void setObject(String parameter, Object x) throws SQLException {
        for (Integer i : getParameterIndexes(parameter)) {
            getDelegate().setObject(i, x);
        }
    }

    private ArrayList parameterValues = new ArrayList();
    private void saveQueryParamValue(int position, Object obj) {
        String strValue;
        if (obj instanceof String || obj instanceof java.util.Date) {
            // if we have a String or Date , include '' in the saved value
            strValue = "'" + obj + "'";
        } else {

            if (obj == null) {
                // convert null to the string null
                strValue = "null";
            } else {
                // unknown object (includes all Numbers), just call toString
                strValue = obj.toString();
            }
        }

        // if we are setting a position larger than current size of parameterValues, first make it larger
        while (position >= parameterValues.size()) {
            parameterValues.add(null);
        }
        // save the parameter
        parameterValues.set(position, strValue);
    }

    public String getQueryString() {

        StringBuffer buf = new StringBuffer();
        try {
            int qMarkCount = 0;

            StringTokenizer tok = new StringTokenizer(sqlTemplate + " ", "?");
            while (tok.hasMoreTokens()) {
                String oneChunk = tok.nextToken();
                buf.append(oneChunk);

                Object value;
                if (parameterValues.size() > 1 + qMarkCount) {
                    value = parameterValues.get(1 + qMarkCount++);
                } else {
                    if (tok.hasMoreTokens()) {
                        value = null;
                    } else {
                        value = "";
                    }
                }
                buf.append("" + value);
            }
        } catch(Throwable ignore){
            buf.append("ERROR WHEN PRODUCING QUERY STRING FOR LOG." + ignore.toString());
            // catch this without whining, if this fails the only thing wrong is probably this class
        }
        return buf.toString().trim();
    }

    @Override
    public ResultSet executeQuery() throws SQLException {
        if( NpPreparedStatement.isQueryLog && log.isDebugEnabled() ){
            log.debug("executeQuery() : " + this.getQueryString());
        }
        return super.executeQuery();
    }

    @Override
    public int executeUpdate() throws SQLException {
        if( NpPreparedStatement.isQueryLog && log.isDebugEnabled() ){
            log.debug("executeUpdate() : " + this.getQueryString());
        }
        return super.executeUpdate();
    }

    @Override
    public ResultSet executeQuery(String sql) throws SQLException {
        if( NpPreparedStatement.isQueryLog && log.isDebugEnabled() ){
            log.debug("executeQuery() : " + this.getQueryString());
        }
        return super.executeQuery(sql);
    }

}
