package hanalife.c.co.service.impl;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import hanalife.c.co.iEnum.MsgEnum;
import hanalife.c.co.mapper.CommCostMapper;
import hanalife.c.co.mapper.dto.StExecCostDTO;
import hanalife.c.co.service.CommCostService;
import hanalife.c.co.util.StringUtil;
import hanalife.c.co.vo.CommCostVO;

/**
 * <pre>
 * 프로젝트명   : 하나생명 가계대출 시스템(DOBSCAP)
 * 클래스명     : CommCostServiceImpl.java Class
 * 클래스설명   : 업무별 공통 Util Service
 * 작성일자     : 2023. 6. 13.
 * 작성자명     : 70319 / 가계대출 시스템 구축
 * Copyrightⓒ2022 HanaLife Insurance, All Rights Reserved.
 * -----------------------------------------------------------------------------
 * Modification Information
 * -----------------------------------------------------------------------------
 * [  수정일자  ][수정자][변경사유-수정내용]
 * -----------------------------------------------------------------------------
 * [2023. 6. 13.][70319 ][최초작성]
 * -----------------------------------------------------------------------------
 * Method Information
 * -----------------------------------------------------------------------------
 * 1. searchCostInfo
 * -----------------------------------------------------------------------------
 * </pre>
 */
@Service("CommCostService")
public class CommCostServiceImpl implements CommCostService {

    private final Logger LOGGER = LoggerFactory.getLogger(CommCostServiceImpl.class);

    /** 공통오류코드 및 오류메시지내용 전역 변수 선언 */
    private MsgEnum msgEnum;

    /** 업무별 공통 Mapper 전역 변수 선언 */
    @Autowired
    private CommCostMapper commCostMapper;

    private int    I_ZERO  = 0;
    private String EXIST_Y = "Y";
    private String EXIST_N = "N";

    /**
     * <pre>
     * 메소드명 	: searchCostInfo
     * 설명 		: 제비용 등록여부 조회
     * 메소드인수1	: 대출신청번호
     * 사용예) CommCostServiceImpl.searchCostInfo(대출신청번호, 데이터구분[M:MCI, H:권원보험, F:전자등기])
     * 메소드리턴값	: SearchLoanSeqVO(대출신청번호, 비용구분, 등록여부, 금액)
     * </pre>
     */
    @Override
    public CommCostVO searchCostInfo(String pstrLoanApplNo, String pstrDataDiv) throws Exception {

        /** 결과값 출력 객체 VO 선언 및 정상 초기값 설정 */
    	CommCostVO resultVo = new CommCostVO();

        resultVo.setRsltCd  (msgEnum.SUCCESS.getErrorCd());
        resultVo.setRsltCntn(msgEnum.SUCCESS.getErrorCn());

        try {

        	// pstrLoanApplNo='111111111111111', pstrDataDiv='00500'
        	CommCostVO inVO = new CommCostVO();

        	inVO.setLoanApplNo  (pstrLoanApplNo);
        	inVO.setTotaMngmGbCd(pstrDataDiv);

        	List<CommCostVO> resultDTO = commCostMapper.selectCostInfo(inVO);

        	int iCustCost       = I_ZERO;
    		int iBankCost       = I_ZERO;
    		int iTotaCostTotAmt = I_ZERO;

        	if( resultDTO != null ) {
        		for (CommCostVO sourceIO : resultDTO) {
        			iCustCost       = sourceIO.getCustCost();
        			iBankCost       = sourceIO.getBankCost();
        			iTotaCostTotAmt = sourceIO.getTotaCostTotAmt();

        			resultVo.setLoanApplNo    (pstrLoanApplNo            );
            		resultVo.setTotaMngmGbCd  (sourceIO.getTotaMngmGbCd());        // 제비용관리구분코드 S11037
            		resultVo.setCustCost      (iCustCost                 );        // 고객부담금액
            		resultVo.setBankCost      (iBankCost                 );        // 은행부담금액
            		resultVo.setTotaCostTotAmt(iTotaCostTotAmt           );        // 제비용부담금액합계
            		resultVo.setBusiCompNo    (sourceIO.getBusiCompNo()  );        // 제비용부담금액합계
            		resultVo.setDataDiv       (sourceIO.getDataDiv()     );        // 데이터업무구분

            		if( iTotaCostTotAmt == 0 ) {
            			resultVo.setCostDivExistYn(EXIST_N      );            // 비용존재여부
            			resultVo.setCostInsDiv    ("U"          );            // 비용등록구분(I:insert, U:update)
            		} else {
            			resultVo.setCostDivExistYn(EXIST_Y      );            // 비용존재여부
            			resultVo.setCostInsDiv    ("X"          );            // 비용등록구분(I:insert, U:update, "X":nothing)
            		}
                }
        	}
        }
        catch ( Exception e ) {
            LOGGER.info(e.toString());

            resultVo.setRsltCd  (msgEnum.ERR_SYSTEM.getErrorCd());
            resultVo.setRsltCntn(e.toString());
        }

        return resultVo;
    }

    /**
     * <pre>
     * 메소드명 	: insertCostInfo
     * 설명 		: 제비용 등록
     * 메소드인수1	: 대출신청번호
     * 사용예) CommCostServiceImpl.insertCostInfo(대출신청번호, 데이터구분[M:MCI, H:권원보험, F:전자등기], 처리구분[I:insert,U:update])
     * 메소드리턴값	: CommCostVO(대출신청번호, 비용구분, 등록여부, 금액)
     * </pre>
     */
    @Override
    public CommCostVO insertCostInfo(String pstrLoanApplNo, String pstrDataDiv, String pstrProcDiv) throws Exception {
    	CommCostVO resultVo = new CommCostVO();

    	resultVo.setRsltCd  (msgEnum.SUCCESS.getErrorCd());
        resultVo.setRsltCntn(msgEnum.SUCCESS.getErrorCn());

        CommCostVO    insVo   = new CommCostVO();

        int iInsertCnt = 0;
        int iUpdateCnt = 0;

        try {
        	insVo.setLoanApplNo(pstrLoanApplNo);
        	insVo.setDataDiv   (pstrDataDiv);

        	if( "I".equals(pstrProcDiv) ) {
        		// insert
        		iInsertCnt = commCostMapper.insertCostInfoList(insVo);
        	} else {
        		// update
        		// 데이터 리스트 조회
        		List<CommCostVO> rtnList = commCostMapper.selectCostInfoList(insVo);

        		if( rtnList != null && rtnList.size() > 0 ) {
        		    for( CommCostVO resultDTO : rtnList ) {
        		    	StExecCostDTO outDTO = new StExecCostDTO();

        		    	outDTO.setLoanApplNo    (resultDTO.getLoanApplNo());
        		    	outDTO.setTotaMngmGbCd  (resultDTO.getTotaMngmGbCd());
        		    	outDTO.setCustCost      (resultDTO.getCustCost());
        		    	outDTO.setBankCost      (resultDTO.getBankCost());
        		    	outDTO.setTotaCostTotAmt(resultDTO.getTotaCostTotAmt());
        		    	outDTO.setBusiCompNo    (StringUtil.null2Str(resultDTO.getBusiCompNo()));

        		    	commCostMapper.updateCostInfo(outDTO);
        		    	iUpdateCnt++;
        		    }
        		}
        	}

        	if( iInsertCnt > 0 || iUpdateCnt > 0 ) {
        		resultVo.setRsltCd  (msgEnum.SUCCESS.getErrorCd());
                resultVo.setRsltCntn(msgEnum.SUCCESS.getDescription());
        	}
        }
        catch ( Exception e ) {
            LOGGER.info(e.toString());

            resultVo.setRsltCd  (msgEnum.ERR_SYSTEM.getErrorCd());
            resultVo.setRsltCntn(e.toString());
        }

    	return resultVo;
    }

}