package hanalife.framework.comm.eai.easymade.connect;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetSocketAddress;
import java.net.Socket;

import javax.annotation.PostConstruct;
import javax.inject.Inject;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Component;

import biz.trustnet.common.util.CommonUtil;
import hanalife.framework.comm.eai.easymade.model.CardBean;
import hanalife.framework.comm.eai.easymade.model.HFBSBean;
import hanalife.framework.comm.eai.easymade.model.MemberBean;
import hanalife.framework.comm.eai.easymade.model.MembershipBean;
import hanalife.framework.comm.util.PropertiesHandler;
import hanalife.util.BizException;

/**
 * PIST 실시간 결제 모듈 Connector (easymadeSoft)
 *
 * @author narui
 * @version 1.0
 * Copyright 2020 HanaI&S co. All right reserved
 *
 *
 * 이력    성 명             일 자                         근 거 자 료                  변 경 내 용
 *    narui  2020.03.03                 실시간 결제 모듈 커넥터
 */
@Component
public class RealTimePayClient {

	private static final Logger logger = LoggerFactory.getLogger(RealTimePayClient.class);

	@Inject
	private Environment env;

	private String ip = "21.98.247.100";
	private final String port = "37971";
	private final String timeOut = "30000";
	private byte[] syncResponse = null;

	/**
	 * 요청 구분
	 */
	public static final String CARD_TYPE_PAY 	= "1";				//카드결제
	public static final String CARD_TYPE_AUTH 	= "3";				//카드인증

	public static final String ACC_TYPE_PAY 	= "A";				//계좌결제(집금)
	public static final String ACC_TYPE_AUTH 	= "E";				//계좌인증
	public static final String ACC_TYPE_CONFIRM = "B";				//계좌(집금)확인(timeout시 재시도)

	public static final String MEM_TYPE_PTSEL 	= "M";				//포인트조회
	public static final String MEM_TYPE_PTPAY 	= "T";				//포인트결제
	public static final String MEM_TYPE_PTCAN 	= "R";				//포인트취소

	public static final String MEM_MCNO			= "HFG5000";		//가맹점 번호
	public static final String MEM_MCNM			= "하나생명대표가맹점";	//가맹점 명

	public static final String MEM_GUBUN		= "2";				//고객_식별_구분_코드 (‘1’- 카드_번호, ‘2’- 멤버십_고객_번호(하나금융 관계사 전용), '3' - CI)

	public static final String MEM_RSN_PAY		= "31001";			//거래사유코드_사용
	public static final String MEM_RSN_CAN		= "21001";			//거래사유코드_취소


	public Socket socket 				= null;
	public InputStream inputStream 		= null;
	public OutputStream outputStream	= null;

	/**
	 * 로드 시 대상 ip 설정
	 */
	@PostConstruct
	public void init() {

		// 2023-04-10 연계 서버 config 정보를 가져오도록 수정
//		String serverMode = env.getProperty("SERVER_MODE");
//		if("R".equals(serverMode)){
//			this.ip = "21.98.249.60";		//운영
//		} else {
//			this.ip = "21.98.247.100";		//UAT
//		}

		String serverMode = PropertiesHandler.getInstance().getProperty("global.current.access.SERVER");

		if(serverMode.equals("PROD")) {
			this.ip = "21.98.249.60";		//운영
		} else {
			this.ip = "21.98.247.100";		//UAT
		}

		logger.info("실시간결제모듈 로드정보(" + serverMode + ") : " + ip);
	}

	/**
	 * 계좌 인증 및 결제
	 * @param hfbsBean
	 * @return
	 * @throws BizException
	 */
	public HFBSBean executeFirm(HFBSBean hfbsBean) throws BizException {

		logger.info("실시간결제모듈(계좌) masked 요청 파라메터:" + hfbsBean.getMaskedToString());

		HFBSBean resBean = null;

		String req = hfbsBean.getTransaction();
		String res = "";

		res = commSync(req, hfbsBean.getTransaction().length(), timeOut);
		logger.info("소켓응답 res:" + res);
		resBean = new HFBSBean(res.getBytes());
		logger.info("HFBSBean Parse masked 소켓응답 res:" + resBean.getMaskedToString());
		return resBean;
	}

	/**
	 * 카드 인증 및 결제
	 * @param cardBean
	 * @return
	 * @throws BizException
	 */
	public CardBean executeCard(CardBean cardBean) throws BizException {

		logger.info("실시간결제모듈(카드) masked 요청 파라메터:" + cardBean.getMaskedToString());

		CardBean resBean = null;

		String req = cardBean.getTransaction();
		String res = "";

		res = commSync(req, cardBean.getTransaction().length(), timeOut);
		logger.info("소켓응답 res:" + res);
		resBean = new CardBean(res.getBytes());
		logger.info("CardBean Parse masked 소켓응답 res:" + resBean.getMaskedToString());
		return resBean;

	}

	/**
	 * 포인트 결제 및 취소
	 * @param mBean
	 * @return
	 * @throws BizException
	 */
	public MembershipBean executeMembership(MembershipBean mBean) throws BizException {

		logger.info("실시간결제모듈(멤버십포인트 결제/취소) 요청 파라메터:" + mBean.toString());

		MembershipBean resBean = null;

		String req = mBean.getTransaction();
		String res = "";

		res = commSync(req, mBean.getTransaction().length(), timeOut);
		logger.info("소켓응답 res:" + res);
		resBean = new MembershipBean(res.getBytes());
		logger.info("MembershipBean Parse 소켓응답 res:" + resBean.toString());
		return resBean;

	}

	/**
	 * 포인트 조회
	 * @param mBean
	 * @return
	 * @throws BizException
	 */
	public MemberBean executeMember(MemberBean mBean) throws BizException {

		logger.info("실시간결제모듈(멤버십포인트 조회) 요청 파라메터:" + mBean.toString());

		MemberBean resBean = null;

		String req = mBean.getTransaction();
		String res = "";

		res = commSync(req, mBean.getTransaction().length(), timeOut);
		logger.info("소켓응답 res:" + res);
		resBean = new MemberBean(res.getBytes());
		logger.info("MemberBean Parse 소켓응답 res:" + resBean.toString());
		return resBean;

	}

	/**
	 * 요청 및 수신
	 * @param request
	 * @param length
	 * @param timeout
	 * @return
	 * @throws BizException
	 */
	public String commSync(String request, int length, String timeout) throws BizException {
		Socket socket 		= null;
    	OutputStream output = null;

    	logger.info("실시간결제모듈 소켓통신 시작합니다, RealTimePayClient.commSync START");

    	boolean error = false;
    	String errorMsg = StringUtils.EMPTY;

    	try{
    		logger.info("CLIENT->SERVER=["+CommonUtil.toString(request)+"]"+" LEN =["+request.length()+"]");

    		connect();
			outputStream = getOutputStream();
			send(request.getBytes());
			inputStream = getInputStream();
			syncResponse = recvAll();

		}catch(Exception e){
			errorMsg = e.getMessage();
			logger.error("실시간결제모듈 소켓통신 중 에러가 발생하였습니다.:" + errorMsg);
			error = true;
		}finally{

			try{
				if(output != null){ output.close();}
				if(socket != null){socket.close();}
			}catch(IOException io){
				logger.error("실시간 인증/결제 완료 후 소켓 닫기 에러:" + io.getMessage());
			}

			if(error){
				throw (new BizException("실시간 인증(결제) 처리시 오류가 발생하였습니다.\n(" + errorMsg + ")"));
			}

			logger.info("CLIENT<-SERVER=["+CommonUtil.toString(syncResponse)+"]"+" LEN =["+CommonUtil.toString(syncResponse.length)+"]");
		}
    	logger.info("실시간결제모듈 소켓통신 종료, RealTimePayClient.commSync END");

    	//return CommonUtil.toString(syncResponse, "MS949"); 로컬통신확인용
    	return CommonUtil.toString(syncResponse);
	}

	/**
	 * 수신
	 * @param requestLength
	 * @return
	 * @throws IOException
	 */
	public byte[] recv(int requestLength)throws IOException{

		byte[] recv = new byte[requestLength];

		boolean run = true;
		int receivedLength = 0;

		while(run){
			byte[] temp = new byte[requestLength-receivedLength];
			int read = inputStream.read(temp);
			arrayCopy(recv, receivedLength,temp,read);
			receivedLength += read;
			if(receivedLength >= requestLength){
				run = false;
			}
		}
		return recv;
	}

	/**
	 * 모두 수신
	 * @return
	 * @throws IOException
	 */
	@SuppressWarnings("unused")
	public byte[] recvAll() throws IOException{

		ByteArrayOutputStream bout = new ByteArrayOutputStream();
        int bcount = 0;
        byte[] buf = new byte[2048];
        int read_retry_count = 0;
        while(true) {
			int n = inputStream.read(buf);
            if ( n > 0 ) { bcount += n; bout.write(buf,0,n); }
            else if (n == -1) break;
            else  {
                if (++read_retry_count >= 5)
                  throw new IOException("inputstream-read-retry-count(5) exceed !");
            }
            if(inputStream.available() == 0){ break; }
        }
        bout.flush();
        byte[] res = bout.toByteArray();
        bout.close();
        return res;

	}

	/**
	 * 전송
	 * @param response
	 * @throws IOException
	 */
	public void send(byte[] response)throws IOException{
		outputStream.write(response,0,response.length);
		outputStream.flush();
	}

	/**
	 * io OutputStream
	 * @return
	 * @throws IOException
	 */
	public OutputStream getOutputStream()throws IOException {
		if(socket == null){
			logger.error("getOutputStream() Error");
			throw new IOException("socket is null");
		}
		return socket.getOutputStream();
	}

	/**
	 * io InputStream
	 * @return
	 * @throws IOException
	 */
	public InputStream getInputStream()throws IOException {
		if(socket == null){
			logger.error("getInputStream() Error");
			throw new IOException("socket is null");
		}
		return socket.getInputStream();
	}

	/**
	 * io 스트림 해제
	 * @throws IOException
	 */
	public void ioClose() throws IOException{
		if(inputStream != null){ inputStream.close();}
		if(outputStream != null){ outputStream.close();}
	}

	/**
	 * 데이터 복사
	 * @param destBuffer
	 * @param destPos
	 * @param srcBuffer
	 * @param srcLen
	 */
	public void arrayCopy(Object destBuffer,int destPos,Object srcBuffer,int srcLen){
		System.arraycopy(srcBuffer,0,destBuffer,destPos,srcLen);
	}


	/**
	 * 소켓 종료
	 */
	public void socketClose(){
		if(socket != null){
			try{
				socket.close();
				socket = null;
			}catch(IOException io){
				io.getStackTrace();
				logger.error("소켓연결 해제 시 에러가 발생하였습니다.:" + io.getMessage());
			}
		}
	}

	/**
	 * 소켓 연결
	 * @throws IOException
	 */
	public void connect() throws IOException{
		try{
			socket = new Socket();
			socket.connect(new InetSocketAddress(ip, CommonUtil.parseInt(port)) , CommonUtil.parseInt(timeOut));

			logger.info("소켓연결 시 성공");

			socket.setSoTimeout(CommonUtil.parseInt(timeOut));
			inputStream = getInputStream();
			outputStream = getOutputStream();
		}catch(IOException io){
			logger.error("소켓연결 시 에러가 발생하였습니다" + io.getMessage());
			io.getStackTrace();
			throw new IOException(io.getMessage());
		}
	}
}
