package hanalife.s.ln.lm.service.impl;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import hanalife.c.co.iEnum.MsgEnum;
import hanalife.c.co.util.DateUtil;
import hanalife.s.ln.lm.mapper.UnrcvProcMapper;
import hanalife.s.ln.lm.mapper.dto.UnrcvOccrAdjDescDTO;
import hanalife.s.ln.lm.mapper.dto.UnrcvOccrBaseDTO;
import hanalife.s.ln.lm.mapper.dto.UnrcvProcDTO;
import hanalife.s.ln.lm.service.UnrcvProcService;
import hanalife.s.ln.lm.vo.UnrcvProcInVO;
import hanalife.s.ln.lm.vo.UnrcvProcOutVO;

/**
 * <pre>
 * 프로젝트명   : 하나생명 가계대출 시스템(DOBSCAP)
 * 클래스명     : UnrcvProcServiceImpl.java Class
 * 클래스설명   : todo1 (UnrcvProc) Service Implements
 * 작성일자     : 2023. 3. 13.
 * 작성자명     : 70324 / younghwe.kim / 가계대출 시스템 구축
 * Copyrightⓒ2022 Hana Life Insurance, All Rights Reserved.
 * -----------------------------------------------------------------------------
 * Modification Information
 * -----------------------------------------------------------------------------
 * [  수정일자  ][수정자][변경사유-수정내용]
 * -----------------------------------------------------------------------------
 * [2023. 3. 13.][70324 ][최초작성]
 * -----------------------------------------------------------------------------
 * 서비스구성 :    rcptUnrcvAmtCancel : 미수금 수납취소처리
 * </pre>
 */
@Service("UnrcvProcService")
public class UnrcvProcServiceImpl implements UnrcvProcService {

	private static final Logger LOGGER = LoggerFactory.getLogger(UnrcvProcServiceImpl.class);

    /** 공통오류코드 및 오류메시지내용 전역 변수 선언 **/
    private MsgEnum msgEnum;

	@Autowired
	private UnrcvProcMapper unrcvProcMapper;

	/**
	 * <p>메소드명      : create : 미수금 생성</p>
	 * <p>작성일자      : 2023. 3. 13.
	 * <p>작성자(author): 70324 / WoongHee.Lee
	 * <p>설명          : 인지세 미수금 생성</p>
	 * <p>메소드인수    : UnrcvProcInVO </p>
	 * <p>메소드리턴값  : UnrcvProcOutVO</p>
	 * <p>예외처리      : Exception</p>
	 */
	@Override
	public UnrcvProcOutVO create(UnrcvProcInVO inVo) throws Exception {

		/** 결과값 출력 객체 VO 선언 및 정상 초기값 설정 **/
		UnrcvProcOutVO resultVo = new UnrcvProcOutVO();
		resultVo.setRsltCd(msgEnum.SUCCESS.getErrorCd());
		resultVo.setRsltCntn(msgEnum.SUCCESS.getErrorCn());

		try {

			/***************************************
             * 입력데이터 체크
             ***************************************/
			if (StringUtils.isBlank(inVo.getLoanNo())) {
	        	resultVo.setRsltCd(msgEnum.ISNULL_LOAN_NO.getErrorCd());
	        	resultVo.setRsltCntn("대출번호를 입력하세요.");
	        	return resultVo;
			}
			if (StringUtils.isBlank(inVo.getLoanSeq())) {
	        	resultVo.setRsltCd(msgEnum.ISNULL_LOAN_SEQ.getErrorCd());
	        	resultVo.setRsltCntn("대출순번을 입력하세요.");
	        	return resultVo;
			}
			if (StringUtils.isBlank(inVo.getUnrcvOccrTpcd())) {
	        	resultVo.setRsltCd(msgEnum.IS_EMPTY.getErrorCd());
	        	resultVo.setRsltCntn("미수 발생유형을 확인하세요.");
	        	return resultVo;
			}
			
			/***************************************
             * ■ 발생정보 입력값 확인
             ***************************************/
			UnrcvOccrBaseDTO validateOccrDto = new UnrcvOccrBaseDTO();
			validateOccrDto.setLoanNo(inVo.getLoanNo());                // 대출번호
			validateOccrDto.setLoanSeq(inVo.getLoanSeq());              // 대출순번
			validateOccrDto.setUnrcvOccrTpcd(inVo.getUnrcvOccrTpcd());  // 미수발생유형코드

	        validateOccr(validateOccrDto);


	        /***************************************
             * ■ 미수기본 생성
             ***************************************/
	        UnrcvOccrBaseDTO unrcvOccrBase = new UnrcvOccrBaseDTO();
	        unrcvOccrBase.setUnrcvNo(inVo.getUnrcvNo());
	        unrcvOccrBase.setLoanNo(inVo.getLoanNo());
	        unrcvOccrBase.setLoanSeq(inVo.getLoanSeq());
	        unrcvOccrBase.setUnrcvOccrDt(inVo.getUnrcvOccrDt());
	        unrcvOccrBase.setUnrcvOccrTpcd(inVo.getUnrcvOccrTpcd());
	        unrcvOccrBase.setUnrcvAdjMeth(inVo.getUnrcvAdjMeth());
	        unrcvOccrBase.setUnrcvOccrAmt(inVo.getUnrcvOccrAmt());
	        unrcvOccrBase.setUnrcvAdjAmt(0);
	        unrcvOccrBase.setUnrcvRamt(inVo.getUnrcvOccrAmt());
	        unrcvOccrBase.setOccrDeptCd(inVo.getOccrDeptCd());
	        unrcvOccrBase.setAdjDeptCd(inVo.getAdjDeptCd());
	        unrcvOccrBase.setFrstRegEmpNo(inVo.getEmpNo());
	        unrcvOccrBase.setLastProcEmpNo(inVo.getEmpNo());
	        unrcvOccrBase = unrcvProcMapper.createUnrcvOccrBase(unrcvOccrBase);


	        /***************************************
	         * ■ 미수내역 생성
	         ***************************************/
	        UnrcvOccrAdjDescDTO unrcvOccrAdjDesc = new UnrcvOccrAdjDescDTO();

	        // ★확인필요?
//	        inVo.setUnrcvAdjTpcd(UnrcvAdjTpcd.UNRCV_OCCR.getCode());

	        unrcvOccrAdjDesc.setUnrcvNo(unrcvOccrBase.getUnrcvNo());
	        unrcvOccrAdjDesc.setLoanNo(unrcvOccrBase.getLoanNo());
	        unrcvOccrAdjDesc.setLoanSeq(unrcvOccrBase.getLoanSeq());
	        unrcvOccrAdjDesc.setUnrcvAdjTpcd(inVo.getUnrcvAdjTpcd());
	        unrcvOccrAdjDesc.setAcctDt(inVo.getAcctDt());
	        unrcvOccrAdjDesc.setSetlDt(inVo.getSetlDt());
	        unrcvOccrAdjDesc.setUnrcvOccrAmt(unrcvOccrBase.getUnrcvOccrAmt());
	        unrcvOccrAdjDesc.setUnrcvAdjAmt(unrcvOccrBase.getUnrcvAdjAmt());
	        unrcvOccrAdjDesc.setUnrcvRamt(unrcvOccrBase.getUnrcvRamt());
	        unrcvOccrAdjDesc.setOccrDeptCd(unrcvOccrBase.getOccrDeptCd());
	        unrcvOccrAdjDesc.setAdjDeptCd(unrcvOccrBase.getAdjDeptCd());
	        unrcvOccrAdjDesc.setFrstRegEmpNo(inVo.getEmpNo());
	        unrcvOccrAdjDesc.setLastProcEmpNo(inVo.getEmpNo());
	        unrcvProcMapper.createUnrcvOccrAdjDesc(unrcvOccrAdjDesc);


            /***************************************
             * ■ 잔액체크
             ***************************************/
            unrcvProcMapper.chkRamt(unrcvOccrBase.getUnrcvNo());


		}catch( Exception e ) {
			// ERR_SYSTEM("888888", "시스템 오류", "시스템 오류가 발생하였습니다. 관리자에게 문의하세요.")
            resultVo.setRsltCd(msgEnum.ERR_SYSTEM.getErrorCd());
            resultVo.setRsltCntn(e.getMessage());
		}

		return resultVo;

	}


	/**
	 * <p>메소드명      : cancel : 취소처리</p>
	 * <p>작성일자      : 2023. 3. 13.
	 * <p>작성자(author): 70324 / WoongHee.Lee
	 * <p>설명          : 인지세 미수금 생성 취소</p>
	 * <p>메소드인수    : UnrcvProcInVO </p>
	 * <p>메소드리턴값  : UnrcvProcOutVO</p>
	 * <p>예외처리      : Exception</p>
	 */
	@Override
	public UnrcvProcOutVO cancel(UnrcvProcInVO inVo) throws Exception {

		/** 결과값 출력 객체 VO 선언 및 정상 초기값 설정 **/
		UnrcvProcOutVO resultVo = new UnrcvProcOutVO();
		resultVo.setRsltCd(msgEnum.SUCCESS.getErrorCd());
		resultVo.setRsltCntn(msgEnum.SUCCESS.getErrorCn());

		try {

			/***************************************
             * ■ 미수원장 확인
             ***************************************/
	        UnrcvOccrBaseDTO searchBase = new UnrcvOccrBaseDTO();
	        searchBase.setUnrcvNo(inVo.getUnrcvNo());
	        UnrcvOccrBaseDTO lastBase = unrcvProcMapper.selectUnrcvOccrBase(searchBase);

	        if ( lastBase == null ) {
                // ERR_SYSTEM("888888", "시스템 오류", "시스템 오류가 발생하였습니다. 관리자에게 문의하세요.")
                resultVo.setRsltCd(msgEnum.NOT_FOUND.getErrorCd());
                resultVo.setRsltCntn("취소 대상이 존재하지 않습니다.");
                return resultVo;
            }

	        if ( lastBase.getUnrcvAdjAmt() > 0 ) {
	        	// ERR_SYSTEM("888888", "시스템 오류", "시스템 오류가 발생하였습니다. 관리자에게 문의하세요.")
	        	resultVo.setRsltCd(msgEnum.NOT_FOUND.getErrorCd());
	        	resultVo.setRsltCntn("미수 정산내역이 존재합니다. 정산취소후 가능합니다.");
	        	return resultVo;
	        }


	        /***************************************
             * ■ 미수잔액 수정
             ***************************************/
	        lastBase.setUnrcvRamt(0);
	        unrcvProcMapper.updateUnrcvRamt(lastBase);


	        /***************************************
             * ■ 미수내역 생성
             ***************************************/
	        UnrcvOccrAdjDescDTO unrcvOccrAdjDesc = new UnrcvOccrAdjDescDTO();
	        unrcvOccrAdjDesc.setUnrcvNo(lastBase.getUnrcvNo());
	        unrcvOccrAdjDesc.setLoanNo(lastBase.getLoanNo());
	        unrcvOccrAdjDesc.setLoanSeq(lastBase.getLoanSeq());

	        // ★확인필요?
//	        unrcvOccrAdjDesc.setUnrcvAdjTpcd(UnrcvAdjTpcd.UNRCV_OCCR_CNCL.getCode());
	        unrcvOccrAdjDesc.setUnrcvAdjTpcd(inVo.getUnrcvAdjTpcd());

	        unrcvOccrAdjDesc.setAcctDt(inVo.getAcctDt());
	        unrcvOccrAdjDesc.setSetlDt(inVo.getSetlDt());
	        unrcvOccrAdjDesc.setUnrcvOccrAmt(-1 * lastBase.getUnrcvOccrAmt());
	        unrcvOccrAdjDesc.setUnrcvAdjAmt(0);
	        unrcvOccrAdjDesc.setUnrcvRamt(0);
	        unrcvOccrAdjDesc.setOccrDeptCd(lastBase.getOccrDeptCd());
	        unrcvOccrAdjDesc.setAdjDeptCd(lastBase.getAdjDeptCd());
	        unrcvOccrAdjDesc.setFrstRegEmpNo(inVo.getEmpNo());
	        unrcvOccrAdjDesc.setLastProcEmpNo(inVo.getEmpNo());
	        unrcvProcMapper.createUnrcvOccrAdjDesc(unrcvOccrAdjDesc);


            /***************************************
             * ■ 잔액체크
             ***************************************/
            unrcvProcMapper.chkRamt(lastBase.getUnrcvNo());


		} catch ( Exception e ) {
			// ERR_SYSTEM("888888", "시스템 오류", "시스템 오류가 발생하였습니다. 관리자에게 문의하세요.")
            resultVo.setRsltCd(msgEnum.ERR_SYSTEM.getErrorCd());
            resultVo.setRsltCntn(e.getMessage());
		}

		return resultVo;

	}


	/**
	 * <p>메소드명      : rcptUnrcvAmt : 미수금 수납처리</p>
	 * <p>작성일자      : 2023. 3. 13.
	 * <p>작성자(author): 70324 / WoongHee.Lee
	 * <p>설명          : 인지세 미수금 수납</p>
	 * <p>메소드인수    : UnrcvProcInVO </p>
	 * <p>메소드리턴값  : UnrcvProcOutVO</p>
	 * <p>예외처리      : Exception</p>
	 */
	@Override
	public UnrcvProcOutVO rcptUnrcvAmt(UnrcvProcInVO inVo) throws Exception {

		/** 결과값 출력 객체 VO 선언 및 정상 초기값 설정 **/
		UnrcvProcOutVO resultVo = new UnrcvProcOutVO();
		resultVo.setRsltCd(msgEnum.SUCCESS.getErrorCd());
		resultVo.setRsltCntn(msgEnum.SUCCESS.getErrorCn());

		try {

			/***************************************
             * ■ 기존 수납건 확인 (대출해약/철회된 대출 미수금 정리는 결제번호 체크 제외)
             ***************************************/
			if ( !"S003".equals(inVo.getUnrcvOccrTpcd()) ) {
				UnrcvProcDTO unrcvProcDTO = new UnrcvProcDTO();
				unrcvProcDTO.setUnrcvNo(inVo.getUnrcvNo());
				unrcvProcDTO.setSetlNo(inVo.getSetlNo());

	    		unrcvProcMapper.chkRcpt(unrcvProcDTO);
	    	}


			/***************************************
             * ■ 미수원장 확인
             ***************************************/
			UnrcvOccrBaseDTO searchBase = new UnrcvOccrBaseDTO();
	        searchBase.setUnrcvNo(inVo.getUnrcvNo());

	        UnrcvOccrBaseDTO lastBase = unrcvProcMapper.selectUnrcvOccrBase(searchBase);

	        if ( lastBase == null ) {
	        	// ERR_SYSTEM("888888", "시스템 오류", "시스템 오류가 발생하였습니다. 관리자에게 문의하세요.")
	        	resultVo.setRsltCd(msgEnum.NOT_FOUND.getErrorCd());
	        	resultVo.setRsltCntn("정산할 미수금정보가 없습니다.");
	        	return resultVo;
	        }
	        if ( lastBase.getUnrcvRamt() < 1 ) {
	        	// ERR_SYSTEM("888888", "시스템 오류", "시스템 오류가 발생하였습니다. 관리자에게 문의하세요.")
	        	resultVo.setRsltCd(msgEnum.NOT_FOUND.getErrorCd());
	        	resultVo.setRsltCntn("정산할 미수금이 없습니다.");
	        	return resultVo;
	        }
	        if ( lastBase.getUnrcvRamt() < inVo.getUnrcvAdjAmt() ) {
	        	// ERR_SYSTEM("888888", "시스템 오류", "시스템 오류가 발생하였습니다. 관리자에게 문의하세요.")
	        	resultVo.setRsltCd(msgEnum.NOT_FOUND.getErrorCd());
	        	resultVo.setRsltCntn("정산금액이 초과입금되었습니다.");
	        	return resultVo;
	        }

	        // □ 대출해약/철회 미수금은 잔액과 정리금액이 일치해야한다.
	        if ( "S003".equals(inVo.getUnrcvOccrTpcd()) ) {
	        	if ( lastBase.getUnrcvRamt() != inVo.getUnrcvAdjAmt() ) {
	        		// ERR_SYSTEM("888888", "시스템 오류", "시스템 오류가 발생하였습니다. 관리자에게 문의하세요.")
		        	resultVo.setRsltCd(msgEnum.NOT_FOUND.getErrorCd());
		        	resultVo.setRsltCntn("정리할 미수금 불일치! 미수금잔액:"+lastBase.getUnrcvRamt()+", 정산금액:"+inVo.getUnrcvAdjAmt());
		        	return resultVo;
	    		}
	        }

	        if ( greaterThan(inVo.getAcctDt(),lastBase.getUnrcvOccrDt()) ) {
	        	// ERR_SYSTEM("888888", "시스템 오류", "시스템 오류가 발생하였습니다. 관리자에게 문의하세요.")
	        	resultVo.setRsltCd(msgEnum.NOT_FOUND.getErrorCd());
	        	resultVo.setRsltCntn("정산일(회계일자)가 미수 발생일자 이전입니다.");
	        	return resultVo;
	        }
	        if ( greaterThan( inVo.getSetlDt(),lastBase.getUnrcvOccrDt()) ) {
	        	// ERR_SYSTEM("888888", "시스템 오류", "시스템 오류가 발생하였습니다. 관리자에게 문의하세요.")
	        	resultVo.setRsltCd(msgEnum.NOT_FOUND.getErrorCd());
	        	resultVo.setRsltCntn("정산일(결제일자)가 미수 발생일자 이전입니다.");
	        	return resultVo;
	        }

	        // ★확인필요?
//	        if ( !RUtil.in( inVo.getUnrcvAdjTpcd(), UnrcvAdjTpcd.RCPT_DEPO.getCode(), UnrcvAdjTpcd.DSRC_TRANS.getCode()) ) {
//	        	// ERR_SYSTEM("888888", "시스템 오류", "시스템 오류가 발생하였습니다. 관리자에게 문의하세요.")
//	        	resultVo.setRsltCd(msgEnum.NOT_FOUND.getErrorCd());
//	        	resultVo.setRsltCntn("정산유형을 확인하세요.(수납정산만 처리가능)");
//	        	return resultVo;
//	        }


	        /***************************************
             * ■ 미수잔액 수정
             * *************************************/
	        lastBase.setUnrcvAdjAmt(lastBase.getUnrcvAdjAmt() + inVo.getUnrcvAdjAmt());
	        lastBase.setUnrcvRamt(lastBase.getUnrcvRamt() - inVo.getUnrcvAdjAmt());
	        unrcvProcMapper.updateUnrcvRamt(lastBase);


	        /***************************************
             * ■ 미수내역 생성
             ***************************************/
	        UnrcvOccrAdjDescDTO unrcvOccrAdjDesc = new UnrcvOccrAdjDescDTO();
	        unrcvOccrAdjDesc.setUnrcvNo(lastBase.getUnrcvNo());
	        unrcvOccrAdjDesc.setLoanNo(lastBase.getLoanNo());
	        unrcvOccrAdjDesc.setLoanSeq(lastBase.getLoanSeq());
	        unrcvOccrAdjDesc.setSetlNo(inVo.getSetlNo());
	        unrcvOccrAdjDesc.setAcctDt(inVo.getAcctDt());
	        unrcvOccrAdjDesc.setSetlDt(inVo.getSetlDt());
	        unrcvOccrAdjDesc.setUnrcvAdjTpcd(inVo.getUnrcvAdjTpcd());
	        unrcvOccrAdjDesc.setUnrcvOccrAmt(lastBase.getUnrcvOccrAmt());    // 유지
	        unrcvOccrAdjDesc.setUnrcvAdjAmt(inVo.getUnrcvAdjAmt());
	        unrcvOccrAdjDesc.setUnrcvRamt(lastBase.getUnrcvRamt());
	        unrcvOccrAdjDesc.setOccrDeptCd(inVo.getOccrDeptCd());
	        unrcvOccrAdjDesc.setAdjDeptCd(inVo.getAdjDeptCd());
	        unrcvOccrAdjDesc.setFrstRegEmpNo(inVo.getEmpNo());
	        unrcvOccrAdjDesc.setLastProcEmpNo(inVo.getEmpNo());

	        unrcvProcMapper.createUnrcvOccrAdjDesc(unrcvOccrAdjDesc);


	        /***************************************
             * ■ 잔액체크
             ***************************************/
	        unrcvProcMapper.chkRamt(lastBase.getUnrcvNo());


		}catch( Exception e ) {
			// ERR_SYSTEM("888888", "시스템 오류", "시스템 오류가 발생하였습니다. 관리자에게 문의하세요.")
            resultVo.setRsltCd(msgEnum.ERR_SYSTEM.getErrorCd());
            resultVo.setRsltCntn(e.getMessage());
		}

		return resultVo;

	}


	/**
	 * <p>메소드명      : procKcarUnrcvAdj : 미수금 정산처리</p>
	 * <p>작성일자      : 2023. 3. 13.
	 * <p>작성자(author): 70324 / WoongHee.Lee
	 * <p>설명          : 미수금 정산처리</p>
	 * <p>메소드인수    : UnrcvProcInVO </p>
	 * <p>메소드리턴값  : UnrcvProcOutVO</p>
	 * <p>예외처리      : Exception</p>
	 */
	@Override
	public UnrcvProcOutVO procKcarUnrcvAdj(UnrcvProcInVO inVo) throws Exception {

        /** 결과값 출력 객체 VO 선언 및 정상 초기값 설정 */
	    UnrcvProcOutVO resultVo = new UnrcvProcOutVO();
        resultVo.setRsltCd(msgEnum.SUCCESS.getErrorCd());
        resultVo.setRsltCntn(msgEnum.SUCCESS.getErrorCn());

        try {
        	String procDt = DateUtil.getServerDate(DateUtil.YYYYMMDD);
        	// 호출 전 반드시 처리해야 할 것
    		// 1) 대출약정원금과 정산금액이 일치한지 체크
    		// 2) 대출상태 42:대출해약/48:철회로 종료된 건인지 체크


    		/***************************************
             * ■ 미수정보 조회
             ***************************************/
        	UnrcvProcDTO unrcvProcDTO = new UnrcvProcDTO();
			unrcvProcDTO.setLoanNo(inVo.getLoanNo());
			unrcvProcDTO.setLoanSeq(inVo.getLoanSeq());
        	UnrcvOccrBaseDTO unrcvBase = unrcvProcMapper.selectKcarUnrcv(unrcvProcDTO);


        	/***************************************
             * ■ 미수금 정산
             ***************************************/
        	UnrcvProcInVO unrcvProcVO = new UnrcvProcInVO();
        	unrcvProcVO.setUnrcvNo(unrcvBase.getUnrcvNo());
			unrcvProcVO.setLoanNo (unrcvBase.getLoanNo());
			unrcvProcVO.setLoanSeq(unrcvBase.getLoanSeq());
			unrcvProcVO.setSetlNo (inVo.getSetlNo());
			unrcvProcVO.setUnrcvAdjAmt(inVo.getUnrcvAdjAmt());

			// ★확인필요?
			unrcvProcVO.setUnrcvAdjTpcd(inVo.getUnrcvAdjTpcd());
//			unrcvProcVO.setUnrcvAdjTpcd(UnrcvAdjTpcd.RCPT_DEPO.getCode());

			unrcvProcVO.setOccrDeptCd(unrcvBase.getOccrDeptCd());
			unrcvProcVO.setAdjDeptCd (unrcvBase.getAdjDeptCd());
			unrcvProcVO.setAcctDt(procDt);
			unrcvProcVO.setSetlDt(procDt);
			unrcvProcVO.setEmpNo(inVo.getEmpNo());
			unrcvProcVO.setUnrcvOccrTpcd(unrcvBase.getUnrcvOccrTpcd());

			this.rcptUnrcvAmt(unrcvProcVO);


        }catch ( Exception e ) {
        	// ERR_SYSTEM("888888", "시스템 오류", "시스템 오류가 발생하였습니다. 관리자에게 문의하세요.")
            resultVo.setRsltCd(msgEnum.ERR_SYSTEM.getErrorCd());
            resultVo.setRsltCntn(e.getMessage());
        }

        return resultVo;
	}


	/**
	 * <p>메소드명      : rcptUnrcvAmtCancel : 미수금 수납취소처리</p>
	 * <p>작성일자      : 2023. 3. 13.
	 * <p>작성자(author): 70324 / WoongHee.Lee
	 * <p>설명          : 미수금 수납취소처리</p>
	 * <p>메소드인수    : UnrcvProcInVO </p>
	 * <p>메소드리턴값  : UnrcvProcOutVO</p>
	 * <p>예외처리      : Exception</p>
	 */
	@Override
	public UnrcvProcOutVO rcptUnrcvAmtCancel(UnrcvProcInVO inVo) throws Exception {

        /** 결과값 출력 객체 VO 선언 및 정상 초기값 설정 */
	    UnrcvProcOutVO resultVo = new UnrcvProcOutVO();
        resultVo.setRsltCd(msgEnum.SUCCESS.getErrorCd());
        resultVo.setRsltCntn(msgEnum.SUCCESS.getErrorCn());

        try {

    		/***************************************
             * ■ 취소처리 정산내역 확인
             ***************************************/
    		UnrcvOccrAdjDescDTO searchDesc = new UnrcvOccrAdjDescDTO();
    		searchDesc.setLoanNo(inVo.getLoanNo());              // 대출번호
    		searchDesc.setLoanSeq(inVo.getLoanSeq());            // 대출순번
    		searchDesc.setSetlNo(inVo.getOrgSetlNo());           //
    		UnrcvOccrAdjDescDTO orgAdjDesc = unrcvProcMapper.selectUnrcvOccrDescForCncl(searchDesc);

    		if ( orgAdjDesc == null ) {
                // ERR_SYSTEM("888888", "시스템 오류", "시스템 오류가 발생하였습니다. 관리자에게 문의하세요.")
                resultVo.setRsltCd(msgEnum.NOT_FOUND.getErrorCd());
                resultVo.setRsltCntn("수납취소할 미수정보가 존재하지 않습니다.(정산내역)");
                return resultVo;
            }

            // ★확인필요?
//            if (!RUtil.in(orgAdjDesc.getUnrcvAdjTpcd(), UnrcvAdjTpcd.RCPT_DEPO.getCode(), UnrcvAdjTpcd.DSRC_TRANS.getCode())) {
//                throw new BizException("수납취소 대상 정산내역을 확인하세요.(수납건이 아닙니다.)");
//            }

            if ( orgAdjDesc.getUnrcvAdjAmt() < 1 ) {
                // ERR_SYSTEM("888888", "시스템 오류", "시스템 오류가 발생하였습니다. 관리자에게 문의하세요.")
                resultVo.setRsltCd(msgEnum.NOT_FOUND.getErrorCd());
                resultVo.setRsltCntn("수납취소 정산금액을 확인하세요.");
                return resultVo;
            }


            /***************************************
             * ■ 미수원장 확인
             ***************************************/
            UnrcvOccrBaseDTO searchBase = new UnrcvOccrBaseDTO();
            searchBase.setUnrcvNo(orgAdjDesc.getUnrcvNo());
            UnrcvOccrBaseDTO unrcvOccrBase = unrcvProcMapper.selectUnrcvOccrBase(searchBase);
            if ( unrcvOccrBase == null ) {
                // ERR_SYSTEM("888888", "시스템 오류", "시스템 오류가 발생하였습니다. 관리자에게 문의하세요.")
                resultVo.setRsltCd(msgEnum.NOT_FOUND.getErrorCd());
                resultVo.setRsltCntn("수납취소할 미수정보가 존재하지 않습니다.(정산내역)");
                return resultVo;
            }


            /***************************************
             * ■ 미수잔액 수정
             ***************************************/
            unrcvOccrBase.setUnrcvAdjAmt(unrcvOccrBase.getUnrcvAdjAmt() - orgAdjDesc.getUnrcvAdjAmt());
            unrcvOccrBase.setUnrcvRamt(unrcvOccrBase.getUnrcvRamt() + orgAdjDesc.getUnrcvAdjAmt());
            unrcvProcMapper.updateUnrcvRamt(unrcvOccrBase);


            /***************************************
             * ■ 미수내역 생성
             ***************************************/
            UnrcvOccrAdjDescDTO unrcvOccrAdjDesc = new UnrcvOccrAdjDescDTO();
            unrcvOccrAdjDesc.setUnrcvNo(unrcvOccrBase.getUnrcvNo());
            unrcvOccrAdjDesc.setLoanNo(unrcvOccrBase.getLoanNo());
            unrcvOccrAdjDesc.setLoanSeq(unrcvOccrBase.getLoanSeq());
            unrcvOccrAdjDesc.setSetlNo(inVo.getSetlNo());

            // ★확인필요?
//            unrcvOccrAdjDesc.setUnrcvAdjTpcd(UnrcvAdjTpcd.getInstanceByCode(orgAdjDesc.getUnrcvAdjTpcd()).getCncl().getCode());
            unrcvOccrAdjDesc.setUnrcvAdjTpcd(inVo.getUnrcvAdjTpcd());

            unrcvOccrAdjDesc.setAcctDt(inVo.getAcctDt());
            unrcvOccrAdjDesc.setSetlDt(inVo.getSetlDt());
            unrcvOccrAdjDesc.setUnrcvOccrAmt(unrcvOccrBase.getUnrcvOccrAmt());   // 유지
            unrcvOccrAdjDesc.setUnrcvAdjAmt(-1 * orgAdjDesc.getUnrcvAdjAmt());
            unrcvOccrAdjDesc.setUnrcvRamt(unrcvOccrBase.getUnrcvRamt());
            unrcvOccrAdjDesc.setOccrDeptCd(inVo.getOccrDeptCd());
            unrcvOccrAdjDesc.setAdjDeptCd(inVo.getAdjDeptCd());
            unrcvOccrAdjDesc.setFrstRegEmpNo(inVo.getEmpNo());
            unrcvOccrAdjDesc.setLastProcEmpNo(inVo.getEmpNo());
            unrcvProcMapper.createUnrcvOccrAdjDesc(unrcvOccrAdjDesc);


            /***************************************
             * ■ 잔액체크
             ***************************************/
            unrcvProcMapper.chkRamt(unrcvOccrBase.getUnrcvNo());


        }catch ( Exception e ) {
            // ERR_SYSTEM("888888", "시스템 오류", "시스템 오류가 발생하였습니다. 관리자에게 문의하세요.")
            resultVo.setRsltCd(msgEnum.ERR_SYSTEM.getErrorCd());
            resultVo.setRsltCntn(e.getMessage());
        }

        return resultVo;

	}


	/**
	 * <p>메소드명      : validateOccr : 발생정보 확인</p>
	 * <p>작성일자      : 2023. 3. 13.
	 * <p>작성자(author): 70324 / WoongHee.Lee
	 * <p>설명          : 발생정보 확인</p>
	 * <p>메소드인수    : UnrcvProcInVO </p>
	 * <p>메소드리턴값  : </p>
	 * <p>예외처리      : Exception</p>
	 */
	private void validateOccr(UnrcvOccrBaseDTO inDto) throws Exception {

		/***************************************
         * ■ 기등록 확인
         ***************************************/
        unrcvProcMapper.chkCreate(inDto);

	}

	private boolean greaterThan(String firstDt, String secondDt) {
        return firstDt.compareTo(secondDt) < 0;
    }


}
