package hanalife.s.sp.as.service.impl;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import hanalife.c.co.contants.Constants;
import hanalife.c.co.iEnum.MsgEnum;
import hanalife.c.co.mapper.dto.StCssIncomeDTO;
import hanalife.c.co.util.StringUtil;
import hanalife.s.sp.as.mapper.SSPASP05Mapper;
import hanalife.s.sp.as.mapper.dto.SSPASP05CHECKSTATCNTDTO;
import hanalife.s.sp.as.service.SSPASP05Service;
import hanalife.s.sp.as.vo.SSPASP05CheckVO;
import hanalife.s.sp.as.vo.SSPASP05DSSTCSSINCOMEVO;
import hanalife.s.sp.as.vo.SSPASP05S01InVO;
import hanalife.s.sp.as.vo.SSPASP05S01OutVO;

/**
 * <pre>
 * 프로젝트명   : 하나생명 가계대출 시스템(DOBSCAP)
 * 클래스명     : SSPASP05ServiceImpl.java Class
 * 클래스설명   : 신청번호검색 (SSPASP05) Service Implements
 * 작성일자     : 2023.04.04.
 * 작성자명     : 70280 / 성관용 / 가계대출 시스템 구축
 * Copyrightⓒ2023 HanaLife Insurance, All Rights Reserved.
 * -----------------------------------------------------------------------------
 * Modification Information
 * -----------------------------------------------------------------------------
 * [  수정일자  ][수정자][변경사유-수정내용]
 * -----------------------------------------------------------------------------
 * [2023.04.04.][70280][최초작성]
 * -----------------------------------------------------------------------------
 * 서비스구성 : getSSPASP05S01 : 연소득 조회
 * -----------------------------------------------------------------------------
 * 소득종류구분코드(S12020)
 * 00010 근로소득
 * 00020 사업소득
 * 00030 연금소득
 * 00040 기타소득
 * 00050 인정소득
 * 00060 신고소득
 *
 * 소득증빙서류코드(S12044)
 * 00101 세무서(홈택스)발급 소득금액증명원(일용직 납세사실증명원) 또는 재형저축소득확인증명서
 * 00102 연말정산용 원천징수 영수증(원천징수부 등 원천징수 증명하는 서류)
 * 00103 급여내역 포함된 증명서(재직회사 날인 급여명세표, 임금대장, 갑근세원천징수확인서)
 * 00201 세무서(홈택스) 발급 소득금액 증명원 또는 재형저축 소득확인증명서
 * 00202 과세표준 확정신고서 및 자진납부 계산사(세무사 확인)
 * 00203 최근년도 연말정산용 사업소득 원천징수 영수증
 * 00204 최근년도 거주자의 사업소득 원천징수 영수증
 * 00301 (연금증서 사본 OR 연금수급권자 확인서) + 연금수령통장(원본확인) 사본(국민,공무원,군인,사학,기타)
 * 00302 연금산정용 가입내역확인서
 * 00303 건강보험료 납부확인서
 * 00401 연말정산용 원천징수 영수증(신용카드 원천징수액)
 * 00402 최근 1년 신용카드 사용 실적표(연말정산 소득공제 비대상 제외)
 *
 * 소득자료제출기간코드(S12045)
 * 000101 개년 소득서류 제출자로 지속가능한 상시소득자(1년이하 재직(사업등))인 차주만 해당
 * 000202 개년도 연소득(환산포함) 비교시 지속 가능한 상시소득자로 최근년도 연소득 적용
 * 000302 개년도 연소득(환산포함) 차이가 ±20% 초과로 평균소득 적용
 * 000401 개년도 소득만 제공하여 10% 차감 (2개년도 소득서류 제출대상이나 일부 미제출)
 * 000501 년이하 소득서류 제출자로 일시소득로 10% 차감"
 * </pre>
 */

@Service("SSPASP05Service")
public class SSPASP05ServiceImpl implements SSPASP05Service
{
    private static final Logger LOGGER = LoggerFactory.getLogger(SSPASP05ServiceImpl.class);

    @Autowired
    private SSPASP05Mapper sSPASP05Mapper;

    /**
     * 응답 오류 정의.
     */
    // 성공
    static final String CONST_RSLT_CD_SUCC                                      = "0000";                // 성공
    static final String CONST_RSLT_CNTN_SUCC                                    = "";                // 성공

    static final String CONST_RSLT_CD_STAT_CNT_SELECT_ERROR                     = "E100";
    static final String CONST_RSLT_CNTN_STAT_CNT_SELECT_ERROR                   = "저장된 소득정보의 상태 조회 중 오류가 발생했습니다.";

    static final String CONST_RSLT_CD_CALC_CVRT_YY_INCO_AMT_ERROR               = "E200";
    static final String CONST_RSLT_CNTN_CALC_CVRT_YY_INCO_AMT_ERROR             = "환산 연소득을 계산할 수 없습니다.\n입력값을 확인하십시오.";

    static final String CONST_RSLT_CD_CHECK_LOAN_CNSL_NO                        = "1101";
    static final String CONST_RSLT_CNTN_CHECK_LOAN_CNSL_NO                      = "대출상담번호가 유효하지 않습니다.\n관리자에게 문의하세요";

    static final String CONST_RSLT_CD_CHECK_PROD_GB_CD                          = "1102";
    static final String CONST_RSLT_CNTN_CHECK_PROD_GB_CD                        = "상품구분코드가 유효하지 않습니다.\n관리자에게 문의하세요";

    static final String CONST_RSLT_CD_CHECK_INCO_OBJ_DIV_CD                     = "1103";
    static final String CONST_RSLT_CNTN_CHECK_INCO_OBJ_DIV_CD                   = "소득대상이 유효하지 않습니다.\n입력값을 확인하십시오.";

    static final String CONST_RSLT_CD_CHECK_INCO_KIND_DIV_CD                    = "1104";
    static final String CONST_RSLT_CNTN_CHECK_INCO_KIND_DIV_CD                  = "소득종류가 유효하지 않습니다.\n입력값을 확인하십시오.";

    static final String CONST_RSLT_CD_CHECK_INCO_PROF_DOC_CD                    = "1105";
    static final String CONST_RSLT_CNTN_CHECK_INCO_PROF_DOC_CD                  = "소득증빙서류가 유효하지 않습니다.\n입력값을 확인하십시오.";

    static final String CONST_RSLT_CD_CHECK_INCO_DATA_SUBM_TERM_CD              = "1106";
    static final String CONST_RSLT_CNTN_CHECK_INCO_DATA_SUBM_TERM_CD            = "소득자료제출기간이 유효하지 않습니다.\n입력값을 확인하십시오.";

    static final String CONST_RSLT_CD_CHECK_RCNT_YY_INCO_AMT                    = "1107";
    static final String CONST_RSLT_CNTN_CHECK_RCNT_YY_INCO_AMT                  = "최근년도 연소득이 유효하지 않습니다.\n입력값을 확인하십시오.";

    static final String CONST_RSLT_CD_CHECK_RCNT_BF_YY_INCO_AMT                 = "1108";
    static final String CONST_RSLT_CNTN_CHECK_RCNT_BF_YY_INCO_AMT               = "최근년도의 전년도 연소득이 유효하지 않습니다.\n입력값을 확인하십시오.";

    static final String CONST_RSLT_CD_SELF_INCOME_FIRST                         = "1201";
    static final String CONST_RSLT_CNTN_SELF_INCOME_FIRST                       = "차주 본인의 소득을 먼저 입력하십시오.";

    static final String CONST_RSLT_CD_SPUS_INCOME_DELETE                        = "1202";
    static final String CONST_RSLT_CNTN_SPUS_INCOME_DELETE                      = "배우자의 소득을 먼저 삭제하십시오.";

    static final String CONST_RSLT_CD_SPUS_INCOME_EXIST                         = "1203";
    static final String CONST_RSLT_CNTN_SPUS_INCOME_EXIST                       = "배우자의 소득이 존재합니다.";

    static final String CONST_RSLT_CD_SELF_INCOME_PROF_EXIST                    = "1301";
    static final String CONST_RSLT_CNTN_SELF_INCOME_PROF_EXIST                  = "차주 본인의 증빙소득이 존재합니다.";

    static final String CONST_RSLT_CD_SELF_INCOME_ADMT_EXIST                    = "1302";
    static final String CONST_RSLT_CNTN_SELF_INCOME_ADMT_EXIST                  = "차주 본인의 인정소득이 존재합니다.";

    static final String CONST_RSLT_CD_SELF_INCOME_DECL_EXIST                    = "1302";
    static final String CONST_RSLT_CNTN_SELF_INCOME_DECL_EXIST                  = "차주 본인의 신고소득이 존재합니다.";

    static final String CONST_RSLT_CD_SPUS_INCOME_PROF_EXIST                    = "2101";
    static final String CONST_RSLT_CNTN_SPUS_INCOME_PROF_EXIST                  = "배우자의 증빙소득이 존재합니다.";

    static final String CONST_RSLT_CD_SPUS_INCOME_ADMT_EXIST                    = "2102";
    static final String CONST_RSLT_CNTN_SPUS_INCOME_ADMT_EXIST                  = "배우자의 인정소득이 존재합니다.";

    static final String CONST_RSLT_CD_SPUS_INCOME_DECL_EXIST                    = "2102";
    static final String CONST_RSLT_CNTN_SPUS_INCOME_DECL_EXIST                  = "배우자의 신고소득이 존재합니다.";

    static final String CONST_RSLT_CD_SELF_INCOME_PROF_SPUS_PROF_REJECT         = "3101";
    static final String CONST_RSLT_CNTN_SELF_INCOME_PROF_SPUS_PROF_REJECT       = "차주 본인의 소득을 증빙소득으로 입력하여,\n배우자 소득은 증빙소득 입력이 불가합니다.";

    static final String CONST_RSLT_CD_SELF_INCOME_PROF_SPUS_ADMT_REJECT         = "3102";
    static final String CONST_RSLT_CNTN_SELF_INCOME_PROF_SPUS_ADMT_REJECT       = "차주 본인의 소득을 증빙소득으로 입력하여,\n배우자 소득은 인정소득 입력이 불가합니다.";

    static final String CONST_RSLT_CD_SELF_INCOME_PROF_SPUS_DECL_REJECT         = "3103";
    static final String CONST_RSLT_CNTN_SELF_INCOME_PROF_SPUS_DECL_REJECT       = "차주 본인의 소득을 증빙소득으로 입력하여,\n배우자 소득은 신고소득 입력이 불가합니다.";

    static final String CONST_RSLT_CD_SELF_INCOME_ADMT_SPUS_PROF_REJECT         = "3201";
    static final String CONST_RSLT_CNTN_SELF_INCOME_ADMT_SPUS_PROF_REJECT       = "차주 본인의 소득을 인정소득으로 입력하여,\n배우자 소득은 증빙소득 입력이 불가합니다.";

    static final String CONST_RSLT_CD_SELF_INCOME_ADMT_SPUS_ADMT_REJECT         = "3202";
    static final String CONST_RSLT_CNTN_SELF_INCOME_ADMT_SPUS_ADMT_REJECT       = "차주 본인의 소득을 인정소득으로 입력하여,\n배우자 소득은 인정소득 입력이 불가합니다.";

    static final String CONST_RSLT_CD_SELF_INCOME_ADMT_SPUS_DECL_REJECT         = "3203";
    static final String CONST_RSLT_CNTN_SELF_INCOME_ADMT_SPUS_DECL_REJECT       = "차주 본인의 소득을 인정소득으로 입력하여,\n배우자 소득은 신고소득 입력이 불가합니다.";

    static final String CONST_RSLT_CD_SELF_INCOME_DECL_SPUS_PROF_REJECT         = "3301";
    static final String CONST_RSLT_CNTN_SELF_INCOME_DECL_SPUS_PROF_REJECT       = "차주 본인의 소득을 신고소득으로 입력하여,\n배우자 소득은 증빙소득 입력이 불가합니다.";

    static final String CONST_RSLT_CD_SELF_INCOME_DECL_SPUS_ADMT_REJECT         = "3302";
    static final String CONST_RSLT_CNTN_SELF_INCOME_DECL_SPUS_ADMT_REJECT       = "차주 본인의 소득을 신고소득으로 입력하여,\n배우자 소득은 인정소득 입력이 불가합니다.";

    static final String CONST_RSLT_CD_SELF_INCOME_DECL_SPUS_DECL_REJECT         = "3303";
    static final String CONST_RSLT_CNTN_SELF_INCOME_DECL_SPUS_DECL_REJECT       = "차주 본인의 소득을 신고소득으로 입력하여,\n배우자 소득은 신고소득 입력이 불가합니다.";


    /**
     * <p>메소드명      : getSSPASP05S01 : 연소득 조회</p>
     * <p>작성일자      : 2023. 3. 22.
     * <p>작성자(author): 70323 / 성관용
     * <p>설명          : 연소득을 조회한다.</p>
     * <p>메소드인수    : SSPASP05S01InVo </p>
     * <p>메소드리턴값  : SSPASP05S01OutVo</p>
     * <p>예외처리      : Exception</p>
     */
    @Override
    public SSPASP05S01OutVO getSSPASP05S02(SSPASP05S01InVO inVo) throws Exception
    {
        LOGGER.debug("CBCCCM01ServiceImpl getCBCCCM01S01 Start");

        SSPASP05S01OutVO resultVo = new SSPASP05S01OutVO();
        resultVo.setRsltCd(MsgEnum.SUCCESS.getErrorCd());
        resultVo.setRsltCntn(MsgEnum.SUCCESS.getDescription());

        String YY_INCO_DECL_SPUS_SUM_LMT = StringUtil.getCnstValue("YY_INCO_DECL_SPUS_SUM_LMT");    // 연소득. 신고소득배우자합산..인정한도.5천만원 (단위:원)
        BigDecimal nYyIncoDeclSpusSumLmt = new BigDecimal(YY_INCO_DECL_SPUS_SUM_LMT);               // 연소득. 신고소득배우자합산..인정한도.5천만원 (단위:원)

        // 연소득 조회 조건 입력값 객체 생성 및 대입 수행
        StCssIncomeDTO oSearchDto = new StCssIncomeDTO();

        oSearchDto.setLoanCnslNo(inVo.getsSPASP05DSSTCSSINCOMEVO().getLoanCnslNo() );  /* 대출상담번호 */
        oSearchDto.setProdGbCd  (inVo.getsSPASP05DSSTCSSINCOMEVO().getProdGbCd()   );  /* 상품구분코드 */

        LOGGER.debug("" + "oSearchDto.getLoanCnslNo() : " + oSearchDto.getLoanCnslNo());
        LOGGER.debug("" + "oSearchDto.getProdGbCd()   : " + oSearchDto.getProdGbCd());

        // 연소득 조회 쿼리 Mapper 호출 수행
        List<StCssIncomeDTO> resultDTO = sSPASP05Mapper.selectStCssIncomeList(oSearchDto);

        // 연소득 화면 출력을 위한 결과값 List VO 객체 생성
        List<SSPASP05DSSTCSSINCOMEVO> resultList = new ArrayList<SSPASP05DSSTCSSINCOMEVO>();
        for (StCssIncomeDTO sourceIO : resultDTO)
        {
            // 연소득 VO에 조회된 Mapper DTO 값을 대입
            SSPASP05DSSTCSSINCOMEVO targetIO = new SSPASP05DSSTCSSINCOMEVO();
            targetIO.setLoanCnslNo              (sourceIO.getLoanCnslNo()         );
            targetIO.setProdGbCd                (sourceIO.getProdGbCd()           );
            targetIO.setIncoObjDivCd            (sourceIO.getIncoObjDivCd()       );
            targetIO.setIncoKindDivCd           (sourceIO.getIncoKindDivCd()      );
            targetIO.setIncoProfDocCd           (sourceIO.getIncoProfDocCd()      );
            targetIO.setIncoDataSubmTermCd      (sourceIO.getIncoDataSubmTermCd() );
            targetIO.setRcntYyIncoAmt           (sourceIO.getRcntYyIncoAmt()      );
            targetIO.setRcntBfYyIncoAmt         (sourceIO.getRcntBfYyIncoAmt()    );
            targetIO.setCvrtYyIncoAmt           (sourceIO.getCvrtYyIncoAmt()      );
            targetIO.setFrstRegDt               (sourceIO.getFrstRegDt()          );   /* */
            targetIO.setFrstRegTm               (sourceIO.getFrstRegTm()          );   /* */
            targetIO.setFrstRegEmpNo            (sourceIO.getFrstRegEmpNo()       );   /* */
            targetIO.setLastProcDt              (sourceIO.getLastProcDt()         );   /* */
            targetIO.setLastProcTm              (sourceIO.getLastProcTm()         );   /* */
            targetIO.setLastProcEmpNo           (sourceIO.getLastProcEmpNo()      );   /* */
            targetIO.setSelfIncoSum             (sourceIO.getSelfIncoSum()        );   /* */
            targetIO.setSpusIncoSum             (sourceIO.getSpusIncoSum()        );   /* */
            targetIO.setYyIncoSumAmt            (sourceIO.getYyIncoSumAmt()       );   /* */

            // 신고소득 합산된 경우, 소득인정금액 제어 (최대 5천만원)
            if (sourceIO.getDeclIncoCnt().compareTo(BigDecimal.ZERO) > 0)
            {
                targetIO.setYyIncoSumAmt        (this.Min(sourceIO.getYyIncoSumAmt(), nYyIncoDeclSpusSumLmt));
            }

            // 연소득 내역조회 List 객체에 추가
            resultList.add(targetIO);
        }

        // 연소득 화면 출력 결과값 대입
        resultVo.setListSSPASP05DSSTCSSINCOMEVO(resultList);

        return resultVo;
    }


    /**
     * <p>메소드명      : insertStCssIncome : 연소득 등록</p>
     * <p>작성일자      : 2023. 3. 22.
     * <p>작성자(author): 70323 / 성관용
     * <p>설명          : 연소득을 등록한다.</p>
     * <p>메소드인수    : SSPASP05S01InVo </p>
     * <p>메소드리턴값  : SSPASP05S01OutVo</p>
     * <p>예외처리      : Exception</p>
     */
    @Override
    public SSPASP05S01OutVO insertStCssIncome(SSPASP05S01InVO inVo) throws Exception
    {
        LOGGER.debug("CBCCCM01ServiceImpl insertStCssIncome Start");

        SSPASP05S01OutVO resultVo = new SSPASP05S01OutVO();
        resultVo.setRsltCd(MsgEnum.SUCCESS.getErrorCd());
        resultVo.setRsltCntn(MsgEnum.SUCCESS.getDescription());

        /*
         * 입력 유효성 체크
         */
        SSPASP05S01OutVO checkVo = null;

        checkVo = this.checkInputValidate(inVo);
        if (checkVo.getRsltCd().equals(MsgEnum.SUCCESS.getErrorCd()) == false)
        {
            resultVo.setRsltCd(checkVo.getRsltCd());
            resultVo.setRsltCntn(checkVo.getRsltCntn());

            return resultVo;
        }

        /*
         * 입력 유효성 체크
         */
        checkVo = this.checkInsertValidate(inVo);
        if (checkVo.getRsltCd().equals(MsgEnum.SUCCESS.getErrorCd()) == false)
        {
            resultVo.setRsltCd(checkVo.getRsltCd());
            resultVo.setRsltCntn(checkVo.getRsltCntn());

            return resultVo;
        }

        /*
         * DTO 설정
         */
        StCssIncomeDTO oInputDto = new StCssIncomeDTO();
        oInputDto.setLoanCnslNo            (inVo.getsSPASP05DSSTCSSINCOMEVO().getLoanCnslNo()         );        // 대출상담번호
        oInputDto.setProdGbCd              (inVo.getsSPASP05DSSTCSSINCOMEVO().getProdGbCd()           );        // 상품구분코드
        oInputDto.setIncoObjDivCd          (inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoObjDivCd()       );        // 소득대상
        oInputDto.setIncoKindDivCd         (inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()      );        // 소득종류
        oInputDto.setIncoProfDocCd         (inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()      );        // 증빙서류
        oInputDto.setIncoDataSubmTermCd    (inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoDataSubmTermCd() );        // 자료제출기간
        oInputDto.setRcntYyIncoAmt         (inVo.getsSPASP05DSSTCSSINCOMEVO().getRcntYyIncoAmt()      );        // 최근년도 연소득
        oInputDto.setRcntBfYyIncoAmt       (inVo.getsSPASP05DSSTCSSINCOMEVO().getRcntBfYyIncoAmt()    );        // 최근년도의 전년도 연소득
        oInputDto.setCvrtYyIncoAmt         (BigDecimal.ZERO                                           );        // 연소득금액 (금액 환산)
        oInputDto.setFrstRegDt             (inVo.getsSPASP05DSSTCSSINCOMEVO().getFrstRegDt()          );        //
        oInputDto.setFrstRegTm             (inVo.getsSPASP05DSSTCSSINCOMEVO().getFrstRegTm()          );        //
        oInputDto.setFrstRegEmpNo          (inVo.getsSPASP05DSSTCSSINCOMEVO().getFrstRegEmpNo()       );        //
        oInputDto.setLastProcDt            (inVo.getsSPASP05DSSTCSSINCOMEVO().getLastProcDt()         );        //
        oInputDto.setLastProcTm            (inVo.getsSPASP05DSSTCSSINCOMEVO().getLastProcTm()         );        //
        oInputDto.setLastProcEmpNo         (inVo.getsSPASP05DSSTCSSINCOMEVO().getLastProcEmpNo()      );        //

        /*
         * 입력값에 따른 불필요 변수 초기화 (쓰레기 데이타 방지)
         */
        // 1개년치 소득 증빙으로 투입된 경우, 최근 직전년도 자료 초기화 (쓰레기 데이타 방지)
        if (Constants.S12045_YY1_INCO.equals(oInputDto.getIncoDataSubmTermCd()) == true
            ||  Constants.S12045_YY1_10P_DOWN.equals(oInputDto.getIncoDataSubmTermCd()) == true
            ||  Constants.S12045_YY1_UNDR.equals(oInputDto.getIncoDataSubmTermCd()) == true)
        {
            oInputDto.setRcntBfYyIncoAmt   (BigDecimal.ZERO);
        }

        // 인정소득 / 신고소득 -> 최근 직전년도 자료 초기화 (쓰레기 데이타 방지)
        if (Constants.S12020_ADMT.equals(oInputDto.getIncoKindDivCd()) == true
            ||  Constants.S12020_DECL.equals(oInputDto.getIncoKindDivCd()) == true)
        {
            oInputDto.setRcntBfYyIncoAmt   (BigDecimal.ZERO);
        }


        /*
         * 환산소득 산출
         * - 증빙소득
         * - 인정소득
         * - 신고소득
         */
        if (oInputDto.getIncoKindDivCd().equals(Constants.S12020_WORK) == true                          // 근로소득
            || oInputDto.getIncoKindDivCd().equals(Constants.S12020_BIZ) == true                        // 사업소득
            || oInputDto.getIncoKindDivCd().equals(Constants.S12020_ANN) == true                        // 연금소득
            || oInputDto.getIncoKindDivCd().equals(Constants.S12020_ETC) == true)                       // 기타소득
        {
            // 증빙소득인 경우 환산 연소득 금액 계산
            calcIncomKindProfCvrtYyIncoAmt(oInputDto);
        }
        else if (oInputDto.getIncoKindDivCd().equals(Constants.S12020_ADMT) == true)                    // 인정소득
        {
            // 인정소득인 경우 환산 연소득 금액 계산
            if (Constants.S12044_ANN.equals(oInputDto.getIncoProfDocCd()) == true)                      // 연금산정용 가입내역확인서
            {
                calcIncomKindAdmtAnnCvrtYyIncoAmt(oInputDto);
            }
            else if (Constants.S12044_INSU.equals(oInputDto.getIncoProfDocCd()) == true)                // 건강보험료 납부확인서
            {
                calcIncomKindAdmtInsuCvrtYyIncoAmt(oInputDto);
            }
        }
        else if (oInputDto.getIncoKindDivCd().equals(Constants.S12020_DECL) == true)                    // 신고소득
        {
            // 신고소득인 경우 환산 연소득 금액 계산
            calcIncomKindDeclCvrtYyIncoAmt(oInputDto);
        }


        /*
         * 환산소득 체크
         */
        checkVo = this.checkCvrtYyIncoAmt(oInputDto);
        if (checkVo.getRsltCd().equals(MsgEnum.SUCCESS.getErrorCd()) == false)
        {
            resultVo.setRsltCd(checkVo.getRsltCd());
            resultVo.setRsltCntn(checkVo.getRsltCntn());

            return resultVo;
        }



        /*
         * 업데이트 & 저장
         */
        StCssIncomeDTO oExistsDto = sSPASP05Mapper.selectStCssIncome(oInputDto);

        if (oExistsDto == null)
        {
            sSPASP05Mapper.insertStCssIncome(oInputDto);
        }
        else
        {
            sSPASP05Mapper.updateStCssIncome(oInputDto);
        }

        return resultVo;
    }


    /**
     * <p>메소드명      : deleteStCssIncome : 연소득 삭제</p>
     * <p>작성일자      : 2023. 3. 22.
     * <p>작성자(author): 70323 / 성관용
     * <p>설명          : 연소득을 삭제한다.</p>
     * <p>메소드인수    : SSPASP05S01InVo </p>
     * <p>메소드리턴값  : SSPASP05S01OutVo</p>
     * <p>예외처리      : Exception</p>
     */
    @Override
    public SSPASP05S01OutVO deleteStCssIncome(SSPASP05S01InVO inVo) throws Exception
    {
        LOGGER.debug("CBCCCM01ServiceImpl deleteStCssIncome Start");

        SSPASP05S01OutVO resultVo = new SSPASP05S01OutVO();
        resultVo.setRsltCd(MsgEnum.SUCCESS.getErrorCd());
        resultVo.setRsltCntn(MsgEnum.SUCCESS.getDescription());

        // 삭제 유효성 체크
        SSPASP05S01OutVO checkVo = this.checkDeleteValidate(inVo);
        if (checkVo.getRsltCd().equals(MsgEnum.SUCCESS.getErrorCd()) == false)
        {
            resultVo.setRsltCd(checkVo.getRsltCd());
            resultVo.setRsltCntn(checkVo.getRsltCntn());

            return resultVo;
        }

        // 저장
        StCssIncomeDTO oInputDto = new StCssIncomeDTO();
        oInputDto.setLoanCnslNo            (inVo.getsSPASP05DSSTCSSINCOMEVO().getLoanCnslNo()         );
        oInputDto.setProdGbCd              (inVo.getsSPASP05DSSTCSSINCOMEVO().getProdGbCd()           );
        oInputDto.setIncoObjDivCd          (inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoObjDivCd()       );
        oInputDto.setIncoKindDivCd         (inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()      );
        oInputDto.setIncoProfDocCd         (inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()      );
        oInputDto.setIncoDataSubmTermCd    (inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoDataSubmTermCd() );
        oInputDto.setRcntYyIncoAmt         (inVo.getsSPASP05DSSTCSSINCOMEVO().getRcntYyIncoAmt()      );
        oInputDto.setRcntBfYyIncoAmt       (inVo.getsSPASP05DSSTCSSINCOMEVO().getRcntBfYyIncoAmt()    );
        oInputDto.setCvrtYyIncoAmt         (inVo.getsSPASP05DSSTCSSINCOMEVO().getCvrtYyIncoAmt()      );
        oInputDto.setFrstRegDt             (inVo.getsSPASP05DSSTCSSINCOMEVO().getFrstRegDt()          );
        oInputDto.setFrstRegTm             (inVo.getsSPASP05DSSTCSSINCOMEVO().getFrstRegTm()          );
        oInputDto.setFrstRegEmpNo          (inVo.getsSPASP05DSSTCSSINCOMEVO().getFrstRegEmpNo()       );
        oInputDto.setLastProcDt            (inVo.getsSPASP05DSSTCSSINCOMEVO().getLastProcDt()         );
        oInputDto.setLastProcTm            (inVo.getsSPASP05DSSTCSSINCOMEVO().getLastProcTm()         );
        oInputDto.setLastProcEmpNo         (inVo.getsSPASP05DSSTCSSINCOMEVO().getLastProcEmpNo()      );

        sSPASP05Mapper.deleteStCssIncome(oInputDto);

        return resultVo;
    }


    /**
     * 입력값 유효성 검사
     * @param inVo
     * @return
     * @throws Exception
     */
    private SSPASP05S01OutVO checkInputValidate(SSPASP05S01InVO inVo) throws Exception
    {
        LOGGER.debug("CBCCCM01ServiceImpl checkInsertValidate Start");

        SSPASP05S01OutVO resultVo = new SSPASP05S01OutVO();
        resultVo.setRsltCd(MsgEnum.SUCCESS.getErrorCd());
        resultVo.setRsltCntn(MsgEnum.SUCCESS.getDescription());

        // 대출상담번호 체크
        if (StringUtils.isEmpty(inVo.getsSPASP05DSSTCSSINCOMEVO().getLoanCnslNo()) == true)
        {
            resultVo.setRsltCd(CONST_RSLT_CD_CHECK_LOAN_CNSL_NO);
            resultVo.setRsltCntn(CONST_RSLT_CNTN_CHECK_LOAN_CNSL_NO);
            return resultVo;
        }

        // 상품구분코드 체크
        if (StringUtils.isEmpty(inVo.getsSPASP05DSSTCSSINCOMEVO().getProdGbCd()) == true)
        {
            resultVo.setRsltCd(CONST_RSLT_CD_CHECK_PROD_GB_CD);
            resultVo.setRsltCntn(CONST_RSLT_CNTN_CHECK_PROD_GB_CD);
            return resultVo;
        }

        // 소득대상 체크
        if (StringUtils.isEmpty(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoObjDivCd()) == true)
        {
            resultVo.setRsltCd(CONST_RSLT_CD_CHECK_INCO_OBJ_DIV_CD);
            resultVo.setRsltCntn(CONST_RSLT_CNTN_CHECK_INCO_OBJ_DIV_CD);
            return resultVo;
        }

        if (Constants.S12043_SELF.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoObjDivCd()) == false
            &&  Constants.S12043_SPUS.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoObjDivCd()) == false)
        {
            resultVo.setRsltCd(CONST_RSLT_CD_CHECK_INCO_OBJ_DIV_CD);
            resultVo.setRsltCntn(CONST_RSLT_CNTN_CHECK_INCO_OBJ_DIV_CD);
            return resultVo;
        }

        // 소득종류 체크
        if (StringUtils.isEmpty(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoObjDivCd()) == true)
        {
            resultVo.setRsltCd(CONST_RSLT_CD_CHECK_INCO_KIND_DIV_CD);
            resultVo.setRsltCntn(CONST_RSLT_CNTN_CHECK_INCO_KIND_DIV_CD);
            return resultVo;
        }

        if (Constants.S12020_WORK.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == false          // 근로소득
            &&  Constants.S12020_BIZ.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == false       // 사업소득
            &&  Constants.S12020_ANN.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == false       // 연금소득
            &&  Constants.S12020_ETC.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == false       // 기타소득
            &&  Constants.S12020_ADMT.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == false      // 인정소득
            &&  Constants.S12020_DECL.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == false)     // 신고소득
        {
            resultVo.setRsltCd(CONST_RSLT_CD_CHECK_INCO_KIND_DIV_CD);
            resultVo.setRsltCntn(CONST_RSLT_CNTN_CHECK_INCO_KIND_DIV_CD);
            return resultVo;
        }

        if (Constants.S12043_SPUS.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoObjDivCd()) == true
            &&  Constants.S12020_ADMT.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)
        {
            // 배우자는 인정소득 입력 금지
            resultVo.setRsltCd(CONST_RSLT_CD_CHECK_INCO_KIND_DIV_CD);
            resultVo.setRsltCntn(CONST_RSLT_CNTN_CHECK_INCO_KIND_DIV_CD);
            return resultVo;
        }

        // 소득증빙서류 체크
        if (StringUtils.isEmpty(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == true)
        {
            resultVo.setRsltCd(CONST_RSLT_CD_CHECK_INCO_PROF_DOC_CD);
            resultVo.setRsltCntn(CONST_RSLT_CNTN_CHECK_INCO_PROF_DOC_CD);
            return resultVo;
        }

        if (Constants.S12044_INCO_AMT_CERT.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false
            && Constants.S12044_YYEND_ADJ.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false
            && Constants.S12044_SALR_SPEC.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false
            && Constants.S12044_BIZ_INCO_AMT_CERT.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false
            && Constants.S12044_TAX_STND.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false
            && Constants.S12044_RCNT_YY_BIZ_INCO.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false
            && Constants.S12044_RCNT_RESI_INCO.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false
            && Constants.S12044_BNKB.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false
            && Constants.S12044_ANN.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false
            && Constants.S12044_INSU.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false
            && Constants.S12044_YYEND_CARD.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false
            && Constants.S12044_CRE_CARD.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false)
        {
            resultVo.setRsltCd(CONST_RSLT_CD_CHECK_INCO_PROF_DOC_CD);
            resultVo.setRsltCntn(CONST_RSLT_CNTN_CHECK_INCO_PROF_DOC_CD);
            return resultVo;
        }

        // 소득종류에 따른 소득증빙서류 정합성 체크
        if (Constants.S12020_WORK.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)             // 근로소득
        {
            if (Constants.S12044_INCO_AMT_CERT.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false
                &&  Constants.S12044_YYEND_ADJ.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false
                &&  Constants.S12044_SALR_SPEC.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false)
            {
                resultVo.setRsltCd(CONST_RSLT_CD_CHECK_INCO_PROF_DOC_CD);
                resultVo.setRsltCntn(CONST_RSLT_CNTN_CHECK_INCO_PROF_DOC_CD);
                return resultVo;
            }
        }
        else if (Constants.S12020_WORK.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)        // 사업소득
        {
            if (Constants.S12044_BIZ_INCO_AMT_CERT.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false
                && Constants.S12044_TAX_STND.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false
                && Constants.S12044_RCNT_YY_BIZ_INCO.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false
                && Constants.S12044_RCNT_RESI_INCO.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false)
            {
                resultVo.setRsltCd(CONST_RSLT_CD_CHECK_INCO_PROF_DOC_CD);
                resultVo.setRsltCntn(CONST_RSLT_CNTN_CHECK_INCO_PROF_DOC_CD);
                return resultVo;
            }
        }
        else if (Constants.S12020_ANN.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)        // 연금소득
        {
            if (Constants.S12044_BNKB.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false)
            {
                resultVo.setRsltCd(CONST_RSLT_CD_CHECK_INCO_PROF_DOC_CD);
                resultVo.setRsltCntn(CONST_RSLT_CNTN_CHECK_INCO_PROF_DOC_CD);
                return resultVo;
            }
        }
        else if (Constants.S12020_ETC.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)        // 기타소득
        {
            if (Constants.S12044_BIZ_INCO_AMT_CERT.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false)
            {
                resultVo.setRsltCd(CONST_RSLT_CD_CHECK_INCO_PROF_DOC_CD);
                resultVo.setRsltCntn(CONST_RSLT_CNTN_CHECK_INCO_PROF_DOC_CD);
                return resultVo;
            }
        }
        else if (Constants.S12020_ADMT.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)        // 인정소득
        {
            if (Constants.S12044_ANN.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false
                && Constants.S12044_INSU.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false)
            {
                resultVo.setRsltCd(CONST_RSLT_CD_CHECK_INCO_PROF_DOC_CD);
                resultVo.setRsltCntn(CONST_RSLT_CNTN_CHECK_INCO_PROF_DOC_CD);
                return resultVo;
            }
        }
        else if (Constants.S12020_DECL.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)        // 신고소득
        {
            if (Constants.S12044_YYEND_CARD.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false
                && Constants.S12044_CRE_CARD.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == false)
            {
                resultVo.setRsltCd(CONST_RSLT_CD_CHECK_INCO_PROF_DOC_CD);
                resultVo.setRsltCntn(CONST_RSLT_CNTN_CHECK_INCO_PROF_DOC_CD);
                return resultVo;
            }
        }

        // 소득자료제출기간 체크
        if (Constants.S12020_WORK.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true          // 근로소득
            ||  Constants.S12020_BIZ.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true       // 사업소득
            ||  Constants.S12020_ANN.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true       // 연금소득
            ||  Constants.S12020_ETC.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)      // 기타소득
        {
            if (StringUtils.isEmpty(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoDataSubmTermCd()) == true)
            {
                resultVo.setRsltCd(CONST_RSLT_CD_CHECK_INCO_DATA_SUBM_TERM_CD);
                resultVo.setRsltCntn(CONST_RSLT_CNTN_CHECK_INCO_DATA_SUBM_TERM_CD);
                return resultVo;
            }
        }

        if (Constants.S12020_ADMT.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true          // 인정소득
            ||  Constants.S12020_DECL.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)     // 기타소득
        {
            if (StringUtils.isEmpty(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoDataSubmTermCd().trim()) == false)
            {
                resultVo.setRsltCd(CONST_RSLT_CD_CHECK_INCO_DATA_SUBM_TERM_CD);
                resultVo.setRsltCntn(CONST_RSLT_CNTN_CHECK_INCO_DATA_SUBM_TERM_CD);
                return resultVo;
            }
        }

        // 최근년도 연소득 체크
        if (BigDecimal.ZERO.compareTo(inVo.getsSPASP05DSSTCSSINCOMEVO().getRcntYyIncoAmt()) >= 0)
        {
            resultVo.setRsltCd(CONST_RSLT_CD_CHECK_RCNT_YY_INCO_AMT);
            resultVo.setRsltCntn(CONST_RSLT_CNTN_CHECK_RCNT_YY_INCO_AMT);
            return resultVo;
        }

        // 최근년도의 전년도 연소득 체크
        if (Constants.S12045_YY2_INCO.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoDataSubmTermCd()) == true
            ||  Constants.S12045_YY2_20P_OVR.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoDataSubmTermCd()) == true)
        {
            if (BigDecimal.ZERO.compareTo(inVo.getsSPASP05DSSTCSSINCOMEVO().getRcntBfYyIncoAmt()) >= 0)
            {
                resultVo.setRsltCd(CONST_RSLT_CD_CHECK_RCNT_BF_YY_INCO_AMT);
                resultVo.setRsltCntn(CONST_RSLT_CNTN_CHECK_RCNT_BF_YY_INCO_AMT);
                return resultVo;
            }
        }

        return resultVo;
    }


    private SSPASP05S01OutVO checkInsertValidate(SSPASP05S01InVO inVo) throws Exception
    {
        LOGGER.debug("CBCCCM01ServiceImpl checkInsertValidate Start");

        SSPASP05S01OutVO resultVo = new SSPASP05S01OutVO();
        resultVo.setRsltCd(MsgEnum.SUCCESS.getErrorCd());
        resultVo.setRsltCntn(MsgEnum.SUCCESS.getDescription());

        // 저장된 소득정보의 상태 조회 (각 조건별 건수)
        SSPASP05CheckVO oCheckStatCntVo = this.getCheckStatCnt(inVo);

        // 소득정보 상태 조회 오류
        // 저장된 소득정보의 상태 조회 중 오류가 발생했습니다.
        if (oCheckStatCntVo == null)
        {
            resultVo.setRsltCd(CONST_RSLT_CD_STAT_CNT_SELECT_ERROR);
            resultVo.setRsltCntn(CONST_RSLT_CD_STAT_CNT_SELECT_ERROR);
            return resultVo;
        }

        // 배우자 소득 있는 상태에서, 본인 소득 입력하는 경우
        // 배우자의 소득이 존재합니다.
        if (Constants.S12043_SELF.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoObjDivCd()))
        {
            if (oCheckStatCntVo.getSpusCnt() > 0)
            {
                resultVo.setRsltCd(CONST_RSLT_CD_SPUS_INCOME_EXIST);
                resultVo.setRsltCntn(CONST_RSLT_CNTN_SPUS_INCOME_EXIST);
                return resultVo;
            }
        }

        // 본인 소득 없는 상태에서, 배우자 소득 입력하는 경우
        // 차주 본인의 소득을 먼저 입력하십시오.
        if (Constants.S12043_SPUS.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoObjDivCd()))
        {
            if (oCheckStatCntVo.getSelfCnt() <= 0)
            {
                resultVo.setRsltCd(CONST_RSLT_CD_SELF_INCOME_FIRST);
                resultVo.setRsltCntn(CONST_RSLT_CNTN_SELF_INCOME_FIRST);
                return resultVo;
            }
        }


        // 본인소득 입력 유효성 체크
        if (Constants.S12043_SELF.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoObjDivCd()))
        {
            // 증빙소득 입력한 경우
            if (Constants.S12020_WORK.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true          // 근로소득
                ||  Constants.S12020_BIZ.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true       // 사업소득
                ||  Constants.S12020_ANN.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true       // 연금소득
                ||  Constants.S12020_ETC.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)      // 기타소득
            {
                // 증빙소득 입력 한 경우. 인정소득 존재
                if (oCheckStatCntVo.getSelfIncoAdmtCnt() > 0)                                                       // 인정소득 건수
                {
                    resultVo.setRsltCd(CONST_RSLT_CD_SELF_INCOME_ADMT_EXIST);
                    resultVo.setRsltCntn(CONST_RSLT_CNTN_SELF_INCOME_ADMT_EXIST);
                    return resultVo;
                }

                // 증빙소득 입력 한 경우. 신고소득 존재
                if (oCheckStatCntVo.getSelfIncoDeclCnt() > 0)                                                       // 신고소득 건수
                {
                    resultVo.setRsltCd(CONST_RSLT_CD_SELF_INCOME_DECL_EXIST);
                    resultVo.setRsltCntn(CONST_RSLT_CNTN_SELF_INCOME_DECL_EXIST);
                    return resultVo;
                }
            }

            // 인정소득 입력한 경우
            if (Constants.S12020_ADMT.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)         // 인정소득
            {
                // 인정소득 입력 한 경우. 증빙소득 존재
                if (oCheckStatCntVo.getSelfIncoWorkCnt() > 0                                                        // 근로소득 건수
                    ||  oCheckStatCntVo.getSelfIncoBizCnt() > 0                                                     // 사업소득 건수
                    ||  oCheckStatCntVo.getSelfIncoAnnCnt() > 0                                                     // 연금소득 건수
                    ||  oCheckStatCntVo.getSelfIncoEtcCnt() > 0)                                                    // 기타소득 건수
                {
                    resultVo.setRsltCd(CONST_RSLT_CD_SELF_INCOME_PROF_EXIST);
                    resultVo.setRsltCntn(CONST_RSLT_CNTN_SELF_INCOME_PROF_EXIST);
                    return resultVo;
                }

                // 인정소득 입력 한 경우. 인정소득 존재
                if (oCheckStatCntVo.getSelfIncoAdmtCnt() > 0)
                {
                    resultVo.setRsltCd(CONST_RSLT_CD_SELF_INCOME_ADMT_EXIST);
                    resultVo.setRsltCntn(CONST_RSLT_CNTN_SELF_INCOME_ADMT_EXIST);
                    return resultVo;
                }

                // 인정소득 입력 한 경우. 신고소득 존재
                if (oCheckStatCntVo.getSelfIncoDeclCnt() > 0)
                {
                    resultVo.setRsltCd(CONST_RSLT_CD_SELF_INCOME_DECL_EXIST);
                    resultVo.setRsltCntn(CONST_RSLT_CNTN_SELF_INCOME_DECL_EXIST);
                    return resultVo;
                }

                /*
                // 인정소득 입력 한 경우. 타 인정소득 존재
                if (Constants.S12044_ANN.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == true) // 국민연금
                {
                    if (oCheckStatCntVo.getSelfIncoAdmtHlthCnt() > 0)                                            // 인정소득.건강보험 건수
                    {
                        resultVo.setRsltCd(CONST_RSLT_CD_SELF_INCOME_ADMT_EXIST);
                        resultVo.setRsltCntn(CONST_RSLT_CNTN_SELF_INCOME_ADMT_EXIST);
                        return resultVo;
                    }
                }

                if (Constants.S12044_INSU.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == true) // 건강보험
                {
                    if (oCheckStatCntVo.getSelfIncoAdmtAnnCnt() > 0)                                            // 인정소득.국민연금 건수
                    {
                        resultVo.setRsltCd(CONST_RSLT_CD_SELF_INCOME_ADMT_EXIST);
                        resultVo.setRsltCntn(CONST_RSLT_CNTN_SELF_INCOME_ADMT_EXIST);
                        return resultVo;
                    }
                }
                */
            }

            // 신고소득 입력한 경우
            if (Constants.S12020_DECL.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)         // 신고소득
            {
                // 신고소득 입력 한 경우. 증빙소득 존재
                if (oCheckStatCntVo.getSelfIncoWorkCnt() > 0                                                        // 근로소득 건수
                    ||  oCheckStatCntVo.getSelfIncoBizCnt() > 0                                                     // 사업소득 건수
                    ||  oCheckStatCntVo.getSelfIncoAnnCnt() > 0                                                     // 연금소득 건수
                    ||  oCheckStatCntVo.getSelfIncoEtcCnt() > 0)                                                    // 기타소득 건수
                {
                    resultVo.setRsltCd(CONST_RSLT_CD_SELF_INCOME_PROF_EXIST);
                    resultVo.setRsltCntn(CONST_RSLT_CNTN_SELF_INCOME_PROF_EXIST);
                    return resultVo;
                }

                // 신고소득 입력 한 경우. 인정소득 존재
                if (oCheckStatCntVo.getSelfIncoAdmtCnt() > 0)                                                       // 인정소득 건수
                {
                    resultVo.setRsltCd(CONST_RSLT_CD_SELF_INCOME_ADMT_EXIST);
                    resultVo.setRsltCntn(CONST_RSLT_CNTN_SELF_INCOME_ADMT_EXIST);
                    return resultVo;
                }

                // 신고소득 입력 한 경우. 신고소득 존재
                if (oCheckStatCntVo.getSelfIncoDeclCnt() > 0)                                                       // 신고소득 건수
                {
                    resultVo.setRsltCd(CONST_RSLT_CD_SELF_INCOME_DECL_EXIST);
                    resultVo.setRsltCntn(CONST_RSLT_CNTN_SELF_INCOME_DECL_EXIST);
                    return resultVo;
                }
            }
        }


        // 배우자의 입력 유효성 체크
        if (Constants.S12043_SPUS.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoObjDivCd()))
        {
            // 증빙소득 입력한 경우
            if (Constants.S12020_WORK.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true          // 근로소득
                ||  Constants.S12020_BIZ.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true       // 사업소득
                ||  Constants.S12020_ANN.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true       // 연금소득
                ||  Constants.S12020_ETC.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)      // 기타소득
            {
                // 증빙소득 입력 한 경우. 인정소득 존재
                if (oCheckStatCntVo.getSpusIncoAdmtCnt() > 0)                                                       // 인정소득 건수
                {
                    resultVo.setRsltCd(CONST_RSLT_CD_SPUS_INCOME_ADMT_EXIST);
                    resultVo.setRsltCntn(CONST_RSLT_CNTN_SPUS_INCOME_ADMT_EXIST);
                    return resultVo;
                }

                // 증빙소득 입력 한 경우. 신고소득 존재
                if (oCheckStatCntVo.getSpusIncoDeclCnt() > 0)                                                       // 신고소득 건수
                {
                    resultVo.setRsltCd(CONST_RSLT_CD_SPUS_INCOME_DECL_EXIST);
                    resultVo.setRsltCntn(CONST_RSLT_CNTN_SPUS_INCOME_DECL_EXIST);
                    return resultVo;
                }
            }

            // 인정소득 입력한 경우
            if (Constants.S12020_ADMT.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)         // 인정소득
            {
                // 인정소득 입력 한 경우. 증빙소득 존재
                if (oCheckStatCntVo.getSpusIncoWorkCnt() > 0                                                        // 근로소득 건수
                    ||  oCheckStatCntVo.getSpusIncoBizCnt() > 0                                                     // 사업소득 건수
                    ||  oCheckStatCntVo.getSpusIncoAnnCnt() > 0                                                     // 연금소득 건수
                    ||  oCheckStatCntVo.getSpusIncoEtcCnt() > 0)                                                    // 기타소득 건수
                {
                    resultVo.setRsltCd(CONST_RSLT_CD_SPUS_INCOME_PROF_EXIST);
                    resultVo.setRsltCntn(CONST_RSLT_CNTN_SPUS_INCOME_PROF_EXIST);
                    return resultVo;
                }

                // 인종소득 입력 한 경우. 인정소득 존재
                if (oCheckStatCntVo.getSpusIncoAdmtCnt() > 0)
                {
                    resultVo.setRsltCd(CONST_RSLT_CD_SPUS_INCOME_DECL_EXIST);
                    resultVo.setRsltCntn(CONST_RSLT_CNTN_SPUS_INCOME_DECL_EXIST);
                    return resultVo;
                }

                // 인정소득 입력 한 경우. 신고소득 존재
                if (oCheckStatCntVo.getSpusIncoDeclCnt() > 0)
                {
                    resultVo.setRsltCd(CONST_RSLT_CD_SPUS_INCOME_DECL_EXIST);
                    resultVo.setRsltCntn(CONST_RSLT_CNTN_SPUS_INCOME_DECL_EXIST);
                    return resultVo;
                }

                /*
                // 인정소득 입력 한 경우. 타 인정소득 존재
                if (Constants.S12044_ANN.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == true) // 국민연금
                {
                    if (oCheckStatCntVo.getSpusIncoAdmtHlthCnt() > 0)                                            // 인정소득.건강보험 건수
                    {
                        resultVo.setRsltCd(CONST_RSLT_CD_SPUS_INCOME_ADMT_EXIST);
                        resultVo.setRsltCntn(CONST_RSLT_CNTN_SPUS_INCOME_ADMT_EXIST);
                        return resultVo;
                    }
                }

                if (Constants.S12044_INSU.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoProfDocCd()) == true) // 건강보험
                {
                    if (oCheckStatCntVo.getSpusIncoAdmtAnnCnt() > 0)                                            // 인정소득.국민연금 건수
                    {
                        resultVo.setRsltCd(CONST_RSLT_CD_SPUS_INCOME_ADMT_EXIST);
                        resultVo.setRsltCntn(CONST_RSLT_CNTN_SPUS_INCOME_ADMT_EXIST);
                        return resultVo;
                    }
                }
                */
            }

            // 신고소득 입력한 경우
            if (Constants.S12020_DECL.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)         // 신고소득
            {
                // 신고소득 입력 한 경우. 증빙소득 존재
                if (oCheckStatCntVo.getSpusIncoWorkCnt() > 0                                                        // 근로소득 건수
                    ||  oCheckStatCntVo.getSpusIncoBizCnt() > 0                                                     // 사업소득 건수
                    ||  oCheckStatCntVo.getSpusIncoAnnCnt() > 0                                                     // 연금소득 건수
                    ||  oCheckStatCntVo.getSpusIncoEtcCnt() > 0)                                                    // 기타소득 건수
                {
                    resultVo.setRsltCd(CONST_RSLT_CD_SPUS_INCOME_PROF_EXIST);
                    resultVo.setRsltCntn(CONST_RSLT_CNTN_SPUS_INCOME_PROF_EXIST);
                    return resultVo;
                }

                // 신고소득 입력 한 경우. 인정소득 존재
                if (oCheckStatCntVo.getSpusIncoAdmtCnt() > 0)                                                       // 인정소득 건수
                {
                    resultVo.setRsltCd(CONST_RSLT_CD_SPUS_INCOME_ADMT_EXIST);
                    resultVo.setRsltCntn(CONST_RSLT_CNTN_SPUS_INCOME_ADMT_EXIST);
                    return resultVo;
                }

                // 신고소득 입력 한 경우. 신고소득 존재
                if (oCheckStatCntVo.getSpusIncoDeclCnt() > 0)                                                       // 신고소득 건수
                {
                    resultVo.setRsltCd(CONST_RSLT_CD_SPUS_INCOME_DECL_EXIST);
                    resultVo.setRsltCntn(CONST_RSLT_CNTN_SPUS_INCOME_DECL_EXIST);
                    return resultVo;
                }
            }
        }


        // 배우자소득 입력 유효성 체크
        if (Constants.S12043_SPUS.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoObjDivCd()))
        {
            // 차주소득을 증빙소득으로 입력한 경우
            if (oCheckStatCntVo.getSelfIncoWorkCnt() > 0                                                        // 근로소득 건수
                ||  oCheckStatCntVo.getSelfIncoBizCnt() > 0                                                     // 사업소득 건수
                ||  oCheckStatCntVo.getSelfIncoAnnCnt() > 0                                                     // 연금소득 건수
                ||  oCheckStatCntVo.getSelfIncoEtcCnt() > 0)                                                    // 기타소득 건수
            {
                // 배우자 소득을 증빙소득으로 입력한 경우
                if (Constants.S12020_WORK.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true          // 근로소득
                    ||  Constants.S12020_BIZ.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true       // 사업소득
                    ||  Constants.S12020_ANN.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true       // 연금소득
                    ||  Constants.S12020_ETC.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)      // 기타소득
                {
                    // 합산 가능하므로 제약 조건 체크하지 않음
                }

                // 배우자 소득을 인정소득으로 입력한 경우
                if (Constants.S12020_ADMT.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)         // 인정소득
                {
                    resultVo.setRsltCd(CONST_RSLT_CD_SELF_INCOME_PROF_SPUS_ADMT_REJECT);
                    resultVo.setRsltCntn(CONST_RSLT_CNTN_SELF_INCOME_PROF_SPUS_ADMT_REJECT);
                    return resultVo;
                }

                // 배우자 소득을 신고소득으로 입력한 경우
                if (Constants.S12020_DECL.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)         // 신고소득
                {
                    // 합산 가능하므로 제약 조건 체크하지 않음
                }
            }

            // 차주 소득을 인정소득으로 입력한 경우
            if (oCheckStatCntVo.getSelfIncoAdmtCnt() > 0)
            {
                // 배우자 소득을 증빙소득으로 입력한 경우
                if (Constants.S12020_WORK.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true          // 근로소득
                    ||  Constants.S12020_BIZ.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true       // 사업소득
                    ||  Constants.S12020_ANN.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true       // 연금소득
                    ||  Constants.S12020_ETC.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)      // 기타소득
                {
                    resultVo.setRsltCd(CONST_RSLT_CD_SELF_INCOME_ADMT_SPUS_PROF_REJECT);
                    resultVo.setRsltCntn(CONST_RSLT_CNTN_SELF_INCOME_ADMT_SPUS_PROF_REJECT);
                    return resultVo;
                }

                // 배우자 소득을 인정소득으로 입력한 경우
                if (Constants.S12020_ADMT.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)         // 인정소득
                {
                    resultVo.setRsltCd(CONST_RSLT_CD_SELF_INCOME_ADMT_SPUS_ADMT_REJECT);
                    resultVo.setRsltCntn(CONST_RSLT_CNTN_SELF_INCOME_ADMT_SPUS_ADMT_REJECT);
                    return resultVo;
                }

                // 배우자 소득을 신고소득으로 입력한 경우
                if (Constants.S12020_DECL.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)         // 신고소득
                {
                    resultVo.setRsltCd(CONST_RSLT_CD_SELF_INCOME_ADMT_SPUS_DECL_REJECT);
                    resultVo.setRsltCntn(CONST_RSLT_CNTN_SELF_INCOME_ADMT_SPUS_DECL_REJECT);
                    return resultVo;
                }
            }

            // 차주소득을 신고소득으로 입력한 경우
            if (oCheckStatCntVo.getSelfIncoDeclCnt() > 0)
            {
                // 배우자 소득을 증빙소득으로 입력한 경우
                if (Constants.S12020_WORK.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true          // 근로소득
                    ||  Constants.S12020_BIZ.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true       // 사업소득
                    ||  Constants.S12020_ANN.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true       // 연금소득
                    ||  Constants.S12020_ETC.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)      // 기타소득
                {
                    // 합산 가능하므로 제약 조건 체크하지 않음
                }

                // 배우자 소득을 인정소득으로 입력한 경우
                if (Constants.S12020_ADMT.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)         // 인정소득
                {
                    resultVo.setRsltCd(CONST_RSLT_CD_SELF_INCOME_DECL_SPUS_ADMT_REJECT);
                    resultVo.setRsltCntn(CONST_RSLT_CNTN_SELF_INCOME_DECL_SPUS_ADMT_REJECT);
                    return resultVo;
                }

                // 배우자 소득을 신고소득으로 입력한 경우
                if (Constants.S12020_DECL.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoKindDivCd()) == true)         // 신고소득
                {
                    // 합산 가능하므로 제약 조건 체크하지 않음
                }
            }
        }

        return resultVo;
    }


    private SSPASP05S01OutVO checkDeleteValidate(SSPASP05S01InVO inVo) throws Exception
    {
        LOGGER.debug("CBCCCM01ServiceImpl checkDeleteValidate Start");

        SSPASP05S01OutVO resultVo = new SSPASP05S01OutVO();
        resultVo.setRsltCd(MsgEnum.SUCCESS.getErrorCd());
        resultVo.setRsltCntn(MsgEnum.SUCCESS.getDescription());

        // 저장된 소득정보의 상태 조회 (각 조건별 건수)
        SSPASP05CheckVO oCheckStatCntVo = this.getCheckStatCnt(inVo);

        // 소득정보 상태 조회 오류
        // 저장된 소득정보의 상태 조회 중 오류가 발생했습니다.
        if (oCheckStatCntVo == null)
        {
            resultVo.setRsltCd(CONST_RSLT_CD_STAT_CNT_SELECT_ERROR);
            resultVo.setRsltCntn(CONST_RSLT_CD_STAT_CNT_SELECT_ERROR);

            return resultVo;
        }

        // 배우자 소득 있는 상태에서, 본인 소득 삭제하는 경우
        if (Constants.S12043_SELF.equals(inVo.getsSPASP05DSSTCSSINCOMEVO().getIncoObjDivCd()))
        {
            if (oCheckStatCntVo.getSpusCnt() > 0)
            {
                resultVo.setRsltCd(CONST_RSLT_CD_SPUS_INCOME_DELETE);
                resultVo.setRsltCntn(CONST_RSLT_CNTN_SPUS_INCOME_DELETE);
                return resultVo;
            }
        }

        return resultVo;
    }


    private SSPASP05CheckVO getCheckStatCnt(SSPASP05S01InVO inVo) throws Exception
    {
        LOGGER.debug("CBCCCM01ServiceImpl getCheckStatCnt Start");

        // 연소득 조회 조건 입력값 객체 생성 및 대입 수행
        StCssIncomeDTO oSearchDto = new StCssIncomeDTO();

        oSearchDto.setLoanCnslNo(inVo.getsSPASP05DSSTCSSINCOMEVO().getLoanCnslNo() );  /* 대출상담번호 */
        oSearchDto.setProdGbCd  (inVo.getsSPASP05DSSTCSSINCOMEVO().getProdGbCd()   );  /* 상품구분코드 */

        // 연소득 조회 쿼리 Mapper 호출 수행
        SSPASP05CHECKSTATCNTDTO oCheckStatCntDto = sSPASP05Mapper.selectCheckStatCnt(oSearchDto);
        if (oCheckStatCntDto == null)
        {
            return null;
        }

        SSPASP05CheckVO oCheckStatCntVo = new SSPASP05CheckVO();

        oCheckStatCntVo.setLoanCnslNo          (oCheckStatCntDto.getLoanCnslNo()              );  // 대출상담번호
        oCheckStatCntVo.setProdGbCd            (oCheckStatCntDto.getProdGbCd()                );  // 상품구분코드
        oCheckStatCntVo.setSelfCnt             (oCheckStatCntDto.getSelfCnt()                 );  // 본인.전체건수
        oCheckStatCntVo.setSpusCnt             (oCheckStatCntDto.getSpusCnt()                 );  // 배우자.전체건수
        oCheckStatCntVo.setSelfIncoWorkCnt     (oCheckStatCntDto.getSelfIncoWorkCnt()         );  // 본인.근로소득
        oCheckStatCntVo.setSelfIncoBizCnt      (oCheckStatCntDto.getSelfIncoBizCnt()          );  // 본인.사업소득
        oCheckStatCntVo.setSelfIncoAnnCnt      (oCheckStatCntDto.getSelfIncoAnnCnt()          );  // 본인.연금소득
        oCheckStatCntVo.setSelfIncoEtcCnt      (oCheckStatCntDto.getSelfIncoEtcCnt()          );  // 본인.기타소득
        oCheckStatCntVo.setSelfIncoAdmtCnt     (oCheckStatCntDto.getSelfIncoAdmtCnt()         );  // 본인.인정소득
        oCheckStatCntVo.setSelfIncoAdmtAnnCnt  (oCheckStatCntDto.getSelfIncoAdmtAnnCnt()      );  // 본인.인정소득.국민연금
        oCheckStatCntVo.setSelfIncoAdmtHlthCnt (oCheckStatCntDto.getSelfIncoAdmtHlthCnt()     );  // 본인.인정소득.건강보험
        oCheckStatCntVo.setSelfIncoDeclCnt     (oCheckStatCntDto.getSelfIncoDeclCnt()         );  // 본인.신고소득
        oCheckStatCntVo.setSpusIncoWorkCnt     (oCheckStatCntDto.getSpusIncoWorkCnt()         );  // 배우자.근로소득
        oCheckStatCntVo.setSpusIncoBizCnt      (oCheckStatCntDto.getSpusIncoBizCnt()          );  // 배우자.사업소득
        oCheckStatCntVo.setSpusIncoAnnCnt      (oCheckStatCntDto.getSpusIncoAnnCnt()          );  // 배우자.연금소득
        oCheckStatCntVo.setSpusIncoEtcCnt      (oCheckStatCntDto.getSpusIncoEtcCnt()          );  // 배우자.기타소득
        oCheckStatCntVo.setSpusIncoAdmtCnt     (oCheckStatCntDto.getSpusIncoAdmtCnt()         );  // 배우자.인정소득
        oCheckStatCntVo.setSpusIncoDeclCnt     (oCheckStatCntDto.getSpusIncoDeclCnt()         );  // 배우자.신고소득

        return oCheckStatCntVo;
    }


    /**
     * 증빙소득 환산 연소득 금액 계산
     * @param oInputDto
     * @return
     */
    private boolean calcIncomKindProfCvrtYyIncoAmt(StCssIncomeDTO oInputDto)
    {
        LOGGER.debug("CBCCCM01ServiceImpl calcIncomKindProfCvrtYyIncoAmt Start");

        if (oInputDto.getIncoDataSubmTermCd().equals(Constants.S12045_YY1_INCO) == true)                    // S12045_YY1_INCO       = "00010";  // 1개년 소득서류 제출자로 지속가능한 상시소득자(1년이하 재직(사업등))인 차주만 해당
        {
            oInputDto.setCvrtYyIncoAmt(oInputDto.getRcntYyIncoAmt());
        }
        else if (oInputDto.getIncoDataSubmTermCd().equals(Constants.S12045_YY2_INCO) == true)               // S12045_YY2_INCO       = "00020";  // 2개년도 연소득(환산포함) 비교시 지속 가능한 상시소득자로 최근년도 연소득 적용
        {
            oInputDto.setCvrtYyIncoAmt(oInputDto.getRcntYyIncoAmt());
        }
        else if (oInputDto.getIncoDataSubmTermCd().equals(Constants.S12045_YY2_20P_OVR) == true)            // S12045_YY2_20P_OVR    = "00030";  // 2개년도 연소득(환산포함) 차이가 ±20% 초과로 평균소득 적용
        {
            BigDecimal nTempDiffAmt = BigDecimal.ZERO;
            nTempDiffAmt = oInputDto.getRcntBfYyIncoAmt().multiply(new BigDecimal("0.2")).setScale(0, RoundingMode.DOWN); // 20%

            BigDecimal nTempUpAmt = oInputDto.getRcntBfYyIncoAmt().add(nTempDiffAmt);
            BigDecimal nTempDownAmt = oInputDto.getRcntBfYyIncoAmt().subtract(nTempUpAmt);
            if ((oInputDto.getRcntYyIncoAmt().compareTo(nTempUpAmt) > 0)  ||  (oInputDto.getRcntYyIncoAmt().compareTo(nTempDownAmt) < 0))
            {
                // 20% 이상 오차 발생 -> 2개년치 평균
                BigDecimal nTempSumAmt = BigDecimal.ZERO;
                nTempSumAmt = nTempSumAmt.add(oInputDto.getRcntYyIncoAmt());                                // 최근년도
                nTempSumAmt = nTempSumAmt.add(oInputDto.getRcntBfYyIncoAmt());                              // 최근년도 직전년도

                nTempSumAmt = nTempSumAmt.divide(new BigDecimal("2")).setScale(0, RoundingMode.DOWN);       // 2개년도 합산
                oInputDto.setCvrtYyIncoAmt(nTempSumAmt);
            }
            else
            {
                // 20% 이내 오차 발생 -> 1개년 치
                oInputDto.setCvrtYyIncoAmt(oInputDto.getRcntYyIncoAmt());
            }
        }
        else if (oInputDto.getIncoDataSubmTermCd().equals(Constants.S12045_YY1_10P_DOWN) == true)          // S12045_YY1_10P_DOWN    = "00040";  // 1개년도 소득만 제공하여 10% 차감 (2개년도 소득서류 제출대상이나 일부 미제출)
        {
            BigDecimal nTempDiffAmt = BigDecimal.ZERO;
            nTempDiffAmt = oInputDto.getRcntYyIncoAmt().multiply(new BigDecimal("0.1")).setScale(0, RoundingMode.DOWN); // 10%

            oInputDto.setCvrtYyIncoAmt(oInputDto.getRcntYyIncoAmt().subtract(nTempDiffAmt));
        }
        else if (oInputDto.getIncoDataSubmTermCd().equals(Constants.S12045_YY1_UNDR) == true)              // S12045_YY1_UNDR        = "00050";  // 1년이하 소득서류 제출자로 일시소득로 10% 차감
        {
            BigDecimal nTempDiffAmt = BigDecimal.ZERO;
            nTempDiffAmt = oInputDto.getRcntYyIncoAmt().multiply(new BigDecimal("0.1")).setScale(0, RoundingMode.DOWN); // 10%

            oInputDto.setCvrtYyIncoAmt(oInputDto.getRcntYyIncoAmt().subtract(nTempDiffAmt));
        }
        else
        {
            oInputDto.setCvrtYyIncoAmt(new BigDecimal("0"));
        }

        return true;
    }


    /**
     * 인정소득 - 국민연금 환산 연소득 금액 계산
     * 연소득 = 최근3개월 평균납부보험류 / 9.000%(보험요율) * 12개월 * 95%
     * @param oInputDto
     * @return
     */
    private boolean calcIncomKindAdmtAnnCvrtYyIncoAmt(StCssIncomeDTO oInputDto)
    {
        LOGGER.debug("CBCCCM01ServiceImpl calcIncomKindAdmtAnnCvrtYyIncoAmt Start");

        String YY_INCO_PPL_ANN_INSU_RT          = StringUtil.getCnstValue("YY_INCO_PPL_ANN_INSU_RT");   // 국민연금 보험요율 (9.000)
        String YY_INCO_ADMT_EXTM_RT             = StringUtil.getCnstValue("YY_INCO_ADMT_EXTM_RT");      // 인정소득 추정 비율 (95)
        String YY_INCO_ADMT_LMT                 = StringUtil.getCnstValue("YY_INCO_ADMT_LMT");          // 인정소득 인정 한도 (5천만원)

        BigDecimal nYyIncoPplAnnInsuRt          = new BigDecimal(YY_INCO_PPL_ANN_INSU_RT);              // 국민연금 보험요율
        BigDecimal nYyIncoAdmtExtmRt            = new BigDecimal(YY_INCO_ADMT_EXTM_RT);                 // 인정소득 추정 비율
        BigDecimal nYyIncoAdmtLmt               = new BigDecimal(YY_INCO_ADMT_LMT);                     // 인정소득 인정한도 (5천만원)

        BigDecimal nMonthCnt                    = new BigDecimal("12");

        BigDecimal nCalcYyIncoPplAnnInsuRt      = nYyIncoPplAnnInsuRt.divide(new BigDecimal("100.0"));  // 9.000%
        BigDecimal nCalcYyIncoAdmtExtmRt        = nYyIncoAdmtExtmRt.divide(new BigDecimal("100.0"));    // 95%

        BigDecimal nRcntYyIncoAmt               = oInputDto.getRcntYyIncoAmt();
        BigDecimal nCvrtYyIncoAmt               = BigDecimal.ZERO;

        // nCvrtYyIncoAmt = nRcntYyIncoAmt.divide(nCalcYyIncoPplAnnInsuRt).multiply(nMonthCnt).multiply(nCalcYyIncoAdmtExtmRt).setScale(0, RoundingMode.DOWN);

        LOGGER.info(">>>>>>>>>>>>>>>>1");
        nCvrtYyIncoAmt = nRcntYyIncoAmt.divide(nCalcYyIncoPplAnnInsuRt, 2, BigDecimal.ROUND_DOWN);
        LOGGER.info(">>>>>>>>>>>>>>>>2");
        nCvrtYyIncoAmt = nCvrtYyIncoAmt.multiply(nMonthCnt);
        LOGGER.info(">>>>>>>>>>>>>>>>3");
        nCvrtYyIncoAmt = nCvrtYyIncoAmt.multiply(nCalcYyIncoAdmtExtmRt);
        LOGGER.info(">>>>>>>>>>>>>>>>4");
        nCvrtYyIncoAmt = nCvrtYyIncoAmt.setScale(0, RoundingMode.DOWN);
        LOGGER.info(">>>>>>>>>>>>>>>>5");

        nCvrtYyIncoAmt = this.Min(nCvrtYyIncoAmt, nYyIncoAdmtLmt);                                      // 인정한도 초과시, 인정한도까지만 인정함.

        oInputDto.setCvrtYyIncoAmt(nCvrtYyIncoAmt);

        return true;
    }


    /**
     * 인정소득 - 건강보험 환산 연소득 금액 계산
     * 연소득 = 최근3개월 평균납부보험류 / 3.545%(보험요율) * 12개월 * 95%
     * @param oInputDto
     * @return
     */
    private boolean calcIncomKindAdmtInsuCvrtYyIncoAmt(StCssIncomeDTO oInputDto)
    {
        LOGGER.debug("CBCCCM01ServiceImpl calcIncomKindAdmtInsuCvrtYyIncoAmt Start");

        String YY_INCO_HLTH_INSU_RT             = StringUtil.getCnstValue("YY_INCO_HLTH_INSU_RT");      // 건강보험 보험요율 (3.545)
        String YY_INCO_ADMT_EXTM_RT             = StringUtil.getCnstValue("YY_INCO_ADMT_EXTM_RT");      // 인정소득 추정 비율 (95)
        String YY_INCO_ADMT_LMT                 = StringUtil.getCnstValue("YY_INCO_ADMT_LMT");          // 인정소득 인정 한도 (5천만원)

        BigDecimal nYyIncoHlthInsuRt            = new BigDecimal(YY_INCO_HLTH_INSU_RT);                 // 건강보험 보험요율
        BigDecimal nYyIncoAdmtExtmRt            = new BigDecimal(YY_INCO_ADMT_EXTM_RT);                 // 인정소득 추정 비율
        BigDecimal nYyIncoAdmtLmt               = new BigDecimal(YY_INCO_ADMT_LMT);                     // 인정소득 인정한도 (5천만원)

        BigDecimal nMonthCnt                    = new BigDecimal("12");

        BigDecimal nCalcYyIncoHlthInsuRt        = nYyIncoHlthInsuRt.divide(new BigDecimal("100.0"));    // 3.545%
        BigDecimal nCalcYyIncoAdmtExtmRt        = nYyIncoAdmtExtmRt.divide(new BigDecimal("100.0"));    // 95%

        BigDecimal nRcntYyIncoAmt               = oInputDto.getRcntYyIncoAmt();
        BigDecimal nCvrtYyIncoAmt               = BigDecimal.ZERO;

        // nCvrtYyIncoAmt = nRcntYyIncoAmt.divide(nCalcYyIncoHlthInsuRt).multiply(nMonthCnt).multiply(nCalcYyIncoAdmtExtmRt).setScale(0, RoundingMode.DOWN);

        LOGGER.info(">>>>>>>>>>>>>>>>1");
        nCvrtYyIncoAmt = nRcntYyIncoAmt.divide(nCalcYyIncoHlthInsuRt, 2, BigDecimal.ROUND_DOWN);
        LOGGER.info(">>>>>>>>>>>>>>>>2");
        nCvrtYyIncoAmt = nCvrtYyIncoAmt.multiply(nMonthCnt);
        LOGGER.info(">>>>>>>>>>>>>>>>3");
        nCvrtYyIncoAmt = nCvrtYyIncoAmt.multiply(nCalcYyIncoAdmtExtmRt);
        LOGGER.info(">>>>>>>>>>>>>>>>4");
        nCvrtYyIncoAmt = nCvrtYyIncoAmt.setScale(0, RoundingMode.DOWN);
        LOGGER.info(">>>>>>>>>>>>>>>>5");

        nCvrtYyIncoAmt = this.Min(nCvrtYyIncoAmt, nYyIncoAdmtLmt);                                      // 인정한도 초과시, 인정한도까지만 인정함.

        oInputDto.setCvrtYyIncoAmt(nCvrtYyIncoAmt);

        return true;
    }


    /**
     * 신고소득 환산 연소득 금액 계산
     * @param oInputDto
     * @return
     */
    private boolean calcIncomKindDeclCvrtYyIncoAmt(StCssIncomeDTO oInputDto)
    {
        LOGGER.debug("CBCCCM01ServiceImpl calcIncomKindDeclCvrtYyIncoAmt Start");

        String YY_INCO_DECL_EXTM_RT             = StringUtil.getCnstValue("YY_INCO_DECL_EXTM_RT");      // 신고소득 추정 비율 (90)
        String YY_INCO_DECL_LMT                 = StringUtil.getCnstValue("YY_INCO_DECL_LMT");          // 신고소득 인정 한도 (5천만원)

        BigDecimal nYyIncoDeclExtmRt            = new BigDecimal(YY_INCO_DECL_EXTM_RT);                 // 신고소득 추정 비율
        BigDecimal nYyIncoDeclLmt               = new BigDecimal(YY_INCO_DECL_LMT);                     // 신고소득 인정한도 (5천만원)

        BigDecimal nCalcYyIncoDeclExtmRt        = nYyIncoDeclExtmRt.divide(new BigDecimal("100.0"));    // 90%

        BigDecimal nRcntYyIncoAmt               = oInputDto.getRcntYyIncoAmt();
        BigDecimal nCvrtYyIncoAmt               = BigDecimal.ZERO;

        nCvrtYyIncoAmt = nRcntYyIncoAmt.multiply(nCalcYyIncoDeclExtmRt).setScale(0, RoundingMode.DOWN);

        nCvrtYyIncoAmt = this.Min(nCvrtYyIncoAmt, nYyIncoDeclLmt);                                      // 인정한도 초과시, 인정한도까지만 인정함.

        oInputDto.setCvrtYyIncoAmt(nCvrtYyIncoAmt);

        return true;
    }


    /**
     * 환산 연소득 금액 정합성 체크
     * @param oInputDto
     * @return
     */
    private SSPASP05S01OutVO checkCvrtYyIncoAmt(StCssIncomeDTO oInputDto) throws Exception
    {
        LOGGER.debug("CBCCCM01ServiceImpl checkCvrtYyIncoAmt Start");

        SSPASP05S01OutVO resultVo = new SSPASP05S01OutVO();
        resultVo.setRsltCd(MsgEnum.SUCCESS.getErrorCd());
        resultVo.setRsltCntn(MsgEnum.SUCCESS.getDescription());

        if (oInputDto.getCvrtYyIncoAmt().compareTo(BigDecimal.ZERO) <= 0)
        {
            resultVo.setRsltCd(CONST_RSLT_CNTN_CALC_CVRT_YY_INCO_AMT_ERROR);
            resultVo.setRsltCntn(CONST_RSLT_CNTN_CALC_CVRT_YY_INCO_AMT_ERROR);
            return resultVo;
        }

        return resultVo;
    }


    private BigDecimal Min(BigDecimal oV1, BigDecimal oV2)
    {
        if (oV1.compareTo(oV2) > 0)
        {
            return oV2;
        }
        else
        {
            return oV1;
        }
    }


    private BigDecimal Max(BigDecimal oV1, BigDecimal oV2)
    {
        if (oV1.compareTo(oV2) > 0)
        {
            return oV1;
        }
        else
        {
            return oV2;
        }
    }
}